@@ -15,7 +15,6 @@ use warnings;
 use LaTeXML::Global;
 use LaTeXML::Common::Object;
 use LaTeXML::Util::Pathname;
-use LaTeXML::Core::Token qw(T_CS);
 use Time::HiRes;
 use Term::ANSIColor qw(colored colorstrip);
 
@@ -38,7 +37,7 @@ our @EXPORT = (
   # Progress Spinner
   qw(&ProgressSpinup &ProgressSpindown &ProgressStep),
   # Debugging messages
-  qw(&DebuggableFeature &Debug &CheckDebuggable),
+  qw(&DebuggableFeature &Debug &CheckDebuggable &Deprecated),
   # Colored-logging related functions
   qw(&colorizeString),
   # stateless message generation
@@ -487,6 +486,13 @@ sub CheckDebuggable {
     print STDERR _freshline(\*STDERR), "Known debugging features: " . join(', ', sort keys %LaTeXML::Debuggable) . "\n"; }
   return; }
 
+sub Deprecated {
+  my ($key, $version, $message) = @_;
+  return if $STATE && $STATE->lookupMapping('DEPRECATIONS_WARNED', $key);
+  Warn('deprecated', $key, undef, "$key is deprecated (as of $version)", $message);
+  $STATE->assignMapping('DEPRECATIONS_WARNED', $key => 1) if $STATE;
+  return; }
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 # Handlers for perl's die & warn
 # We'll try to decode some common errors to make them more usable
