@@ -90,30 +90,31 @@ sub new {
   my $self = bless {    # table => {},
     value   => {}, meaning  => {}, stash  => {}, stash_active => {},
     catcode => {}, mathcode => {}, sfcode => {}, lccode       => {}, uccode => {}, delcode => {},
-    undo => [{ _FRAME_LOCK_ => 1 }], prefixes => {}, status => {},
-    stomach => $options{stomach}, model => $options{model} }, $class;
+    undo    => [{ _FRAME_LOCK_ => 1 }], prefixes => {}, status => {},
+    stomach => $options{stomach},       model    => $options{model} }, $class;
   # Note that "100" is hardwired into TeX, The Program!!!
   $$self{value}{MAX_ERRORS} = [100];
-  $$self{value}{VERBOSITY}  = [0];
   # Standard TeX units, in scaled points
   $$self{value}{UNITS} = [{
-      pt => 65536, pc => 12 * 65536, in => 72.27 * 65536, bp => 72.27 * 65536 / 72,
+      pt => 65536,                    pc => 12 * 65536, in => 72.27 * 65536, bp => 72.27 * 65536 / 72,
       cm => 72.27 * 65536 / 2.54,     mm => 72.27 * 65536 / 2.54 / 10, dd => 1238 * 65536 / 1157,
-      cc => 12 * 1238 * 65536 / 1157, sp => 1 }];
+      cc => 12 * 1238 * 65536 / 1157, sp => 1,
+      px => 72.27 * 65536 / 72,    # Assume px=bp ?
+  }];
 
   $options{catcodes} = 'standard' unless defined $options{catcodes};
   if ($options{catcodes} =~ /^(standard|style)/) {
     # Setup default catcodes.
     my %std = ("\\" => CC_ESCAPE, "{" => CC_BEGIN, "}" => CC_END, "\$" => CC_MATH,
-      "\&" => CC_ALIGN, "\r" => CC_EOL,   "#"  => CC_PARAM, "^" => CC_SUPER,
-      "_"  => CC_SUB,   " "  => CC_SPACE, "\t" => CC_SPACE, "%" => CC_COMMENT,
-      "~" => CC_ACTIVE, chr(0) => CC_IGNORE, "\f" => CC_ACTIVE);
+      "&" => CC_ALIGN,  "\r"   => CC_EOL,    "#"  => CC_PARAM, "^" => CC_SUPER,
+      "_" => CC_SUB,    " "    => CC_SPACE,  "\t" => CC_SPACE, "%" => CC_COMMENT,
+      "~" => CC_ACTIVE, chr(0) => CC_ESCAPE, "\f" => CC_ACTIVE);
     map { $$self{catcode}{$_} = [$std{$_}] } keys %std;
     for (my $c = ord('A') ; $c <= ord('Z') ; $c++) {
       $$self{catcode}{ chr($c) } = [CC_LETTER];
       $$self{catcode}{ chr($c + ord('a') - ord('A')) } = [CC_LETTER]; }
   }
-  $$self{value}{SPECIALS} = [['^', '_', '@', '~', '&', '$', '#', "'"]];
+  $$self{value}{SPECIALS} = [['^', '_', '~', '&', '$', '#', "'"]];
   if ($options{catcodes} eq 'style') {
     $$self{catcode}{'@'} = [CC_LETTER]; }
   $$self{mathcode}            = {};
@@ -126,10 +127,19 @@ sub new {
 
 sub assign_internal {
   my ($self, $table, $key, $value, $scope) = @_;
+  # hotcode lookupDefinition for \globaldefs,
+  # since this is called extremely often and should be highly standardized
+  if (my $globaldefs = $$self{value}{'\globaldefs'}) {
+    if (my $global_value = $$globaldefs[0][0]) {
+      # magic TeX register override: \globaldefs
+      if ($global_value == 1) {
+        $scope = 'global'; }
+      elsif ($global_value == -1) {
+        $scope = 'local'; } } }
   $scope = ($$self{prefixes}{global} ? 'global' : 'local') unless defined $scope;
   if (exists $$self{tracing_definitions}{$key}) {
-    print STDERR "ASSIGN $key in $table " . ($scope ? "($scope)" : '') . " => " .
-      (ref $value ? $value->stringify : $value) . "\n"; }
+    Debug("ASSIGN $key in $table " . ($scope ? "($scope)" : '') . " => " .
+        (ref $value ? $value->stringify : $value)); }
   if ($scope eq 'global') {
     # Remove bindings made in all frames down-to & including the next lower locked frame
     my $frame;
@@ -150,7 +160,6 @@ sub assign_internal {
       $$self{undo}[0]{$table}{$key} = 1;
       unshift(@{ $$self{$table}{$key} }, $value); } }    # And push new binding.
   else {
-    # print STDERR "Assigning $key in stash $stash\n";
     assign_internal($self, 'stash', $scope, [], 'global') unless $$self{stash}{$scope}[0];
     push(@{ $$self{stash}{$scope}[0] }, [$table, $key, $value]);
     assign_internal($self, $table, $key, $value, 'local')
@@ -318,27 +327,26 @@ sub assignDelcode {
 #======================================================================
 # Specialized versions of lookup & assign for dealing with definitions
 
-our @active_or_cs = (
+our @CATCODE_ACTIVE_OR_CS = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 1, 0, 0,
-  1, 0);
-our @letter_or_other = (
+  1, 0, 0, 0);
+our @CATCODE_LETTER_OR_OTHER = (
   0, 0, 0, 0,
   0, 0, 0, 0,
   0, 0, 0, 1,
   1, 0, 0, 0,
-  0, 0);
+  0, 0, 0, 0);
 
 # Get the `Meaning' of a token.  For active control sequence's
 # this may give the definition object (if defined) or another token (if \let) or undef
 # Any other token is returned as is.
 sub lookupMeaning {
   my ($self, $token) = @_;
-  my $e;
   if (my $cs = $token
-    && $active_or_cs[$$token[1]]
+    && $CATCODE_ACTIVE_OR_CS[$$token[1]]
     && $$token[0]) {
     my $e = $$self{meaning}{$cs}; return $e && $$e[0]; }
   else { return $token; } }
@@ -356,25 +364,25 @@ sub assignMeaning {
 # nor cs let to executable tokens
 # This returns a definition object, or undef
 
-# merge of @executable_catcode & @PRIMITIVE_NAME
-our @executable_primitive_name = (    # [CONSTANT]
+# merge of @CATCODE_EXECUTABLE & @CATCODE_PRIMITIVE_NAME
+our @CATCODE_EXECUTABLE_PRIMITIVE_NAME = (    # [CONSTANT]
   undef,       'Begin', 'End', 'Math',
   'Align',     undef,   undef, 'Superscript',
   'Subscript', undef,   undef, undef,
   undef,       undef,   undef, undef,
-  undef,       undef);
+  undef,       undef,   undef, undef);
 
 sub lookupDefinition {
   my ($self, $token) = @_;
   return unless $token;
   my $defn;
   my $entry;
-  #  my $inmath = $self->lookupValue('IN_MATH');
+  #  my $inmath = lookupValue($self, 'IN_MATH');
   my $cc = $$token[1];
   my $lookupname =
-    ($active_or_cs[$cc]
+    ($CATCODE_ACTIVE_OR_CS[$cc]
     ? $$token[0]
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname
     && ($entry = $$self{meaning}{$lookupname})
     && ($defn  = $$entry[0])
@@ -389,14 +397,8 @@ sub lookupConditional {
   return unless $token;
   my $defn;
   my $entry;
-  #  my $inmath = $self->lookupValue('IN_MATH');
-  my $cc = $$token[1];
-  my $lookupname =
-    ($active_or_cs[$cc]
-    ? $$token[0]
-    : $executable_primitive_name[$cc]);
-  if ($lookupname
-    && ($entry = $$self{meaning}{$lookupname})
+  if ($CATCODE_ACTIVE_OR_CS[$$token[1]]
+    && ($entry = $$self{meaning}{ $$token[0] })
     && ($defn  = $$entry[0])
     # Can only be a token or definition; we only want defns that have conditional_type
     && ((ref $defn) ne 'LaTeXML::Core::Token')) {
@@ -409,22 +411,36 @@ sub lookupExpandable {
   return unless $token;
   my $defn;
   my $entry;
-  #  my $inmath = $self->lookupValue('IN_MATH');
-  my $cc = $$token[1];
-  my $lookupname =
-    ($active_or_cs[$cc]
-    ? $$token[0]
-    : $executable_primitive_name[$cc]);
-  if ($lookupname
-    && ($entry = $$self{meaning}{$lookupname})
+  if ($CATCODE_ACTIVE_OR_CS[$$token[1]]
+    && ($entry = $$self{meaning}{ $$token[0] })
     && ($defn  = $$entry[0])
     # Can only be a token or definition; we want defns!
     && ((ref $defn) ne 'LaTeXML::Core::Token')
-    && $$defn{isExpandable}
+    && $defn->isExpandable
     && ($toplevel || !$$defn{isProtected})) { # is this the right logic here? don't expand unless digesting?
     return $defn; }
   return; }
 
+# Whether token is affected by \noexpand
+sub isDontExpandable {
+  my ($self, $token) = @_;
+  # Basically: a CS or Active token that is either not defined, or is expandable
+  # (but not \let to a token)
+  return unless $token;
+  my $defn;
+  my $entry;
+  #  my $inmath = lookupValue($self, 'IN_MATH');
+  my $cc = $$token[1];
+  if ($CATCODE_ACTIVE_OR_CS[$cc]) {
+    my $lookupname = $$token[0];
+    if ($lookupname
+      && ($entry = $$self{meaning}{$lookupname})
+      && ($defn  = $$entry[0])) {
+      return ((ref $defn) ne 'LaTeXML::Core::Token') && $defn->isExpandable; }
+    else {
+      return 1; } }
+  return; }
+
 # used for digestion
 # This recognizes mathactive tokens in math mode
 # and also looks for cs that have been let to other `executable' tokens
@@ -435,23 +451,26 @@ sub lookupDigestableDefinition {
   return unless $token;
   my $defn;
   my $entry;
-  #  my $inmath = $self->lookupValue('IN_MATH');
+  #  my $inmath = lookupValue($self, 'IN_MATH');
   my $cc   = $$token[1];
   my $name = $$token[0];
   my $lookupname =
-    (($active_or_cs[$cc]
-        || ($letter_or_other[$cc] && $self->lookupValue('IN_MATH')
-        && (($self->lookupMathcode($name) || 0) == 0x8000)))
+    (($CATCODE_ACTIVE_OR_CS[$cc]
+        || ($CATCODE_LETTER_OR_OTHER[$cc] && lookupValue($self, 'IN_MATH')
+        && ((lookupMathcode($self, $name) || 0) == 0x8000)))
     ? $name
-    : $executable_primitive_name[$cc]);
+    : $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$cc]);
   if ($lookupname && ($entry = $$self{meaning}{$lookupname})
     && ($defn = $$entry[0])) {
     # If a cs has been let to an executable token, lookup ITS defn.
     if (((ref $defn) eq 'LaTeXML::Core::Token')
-      && ($lookupname = $executable_primitive_name[$$defn[1]])
+      # If we're digesting an unexpanded, act like \relax
+      && ($lookupname = $CATCODE_EXECUTABLE_PRIMITIVE_NAME[$$defn[1]])
       && ($entry      = $$self{meaning}{$lookupname})) {
       $defn = $$entry[0]; }
     return $defn; }
+  elsif ($CATCODE_ACTIVE_OR_CS[$cc]) {
+    return; }
   return $token; }
 
 # And a shorthand for installing definitions
@@ -461,13 +480,13 @@ sub installDefinition {
   # Ignore attempts to (re)define $cs from tex sources
   #  my $cs = $definition->getCS->getCSName;
   my $token = $definition->getCS;
-  my $cs    = ($LaTeXML::Core::Token::PRIMITIVE_NAME[$$token[1]] || $$token[0]);
-  if ($self->lookupValue("$cs:locked") && !$LaTeXML::Core::State::UNLOCKED) {
-    my $s = $self->getStomach->getGullet->getSource;
+  my $cs    = ($LaTeXML::Core::Token::CATCODE_PRIMITIVE_NAME[$$token[1]] || $$token[0]);
+  if (lookupValue($self, "$cs:locked") && !$LaTeXML::Core::State::UNLOCKED) {
+    my $s = $$self{stomach}->getGullet->getSource;
     # report if the redefinition seems to come from document source
     if (((!defined($s)) || ($s =~ /\.(tex|bib)$/))
       && ($s !~ /\.code\.tex$/)) {
-      Info('ignore', $cs, $self->getStomach, "Ignoring redefinition of $cs"); }
+      Info('ignore', $cs, $$self{stomach}, "Ignoring redefinition of $cs"); }
     return; }
   assign_internal($self, 'meaning', $cs => $definition, $scope);
   return; }
@@ -483,6 +502,33 @@ sub installDefinition {
 
 #======================================================================
 
+# Generate a stub definition for an undefined control-sequence,
+# along with appropriate error messge.
+sub generateErrorStub {
+  my ($self, $caller, $token, $params) = @_;
+  my $cs = $token->getCSName;
+  noteStatus($self, undefined => $cs);
+  # To minimize chatter, go ahead and define it...
+  if ($cs =~ /^\\if(.*)$/) {    # Apparently an \ifsomething ???
+    my $name = $1;
+    Error('undefined', $token, $caller, "The token " . $token->stringify . " is not defined.",
+      "Defining it now as with \\newif");
+    installDefinition($self, LaTeXML::Core::Definition::Expandable->new(
+        T_CS('\\' . $name . 'true'), undef, '\let' . $cs . '\iftrue'));
+    installDefinition($self, LaTeXML::Core::Definition::Expandable->new(
+        T_CS('\\' . $name . 'false'), undef, '\let' . $cs . '\iffalse'));
+    LaTeXML::Package::Let($token, T_CS('\iffalse')); }
+  else {
+    Error('undefined', $token, $caller, "The token " . $token->stringify . " is not defined.",
+      "Defining it now as <ltx:ERROR/>");
+    installDefinition($self, LaTeXML::Core::Definition::Constructor->new($token, $params,
+        sub { $_[0]->makeError('undefined', $cs); },
+        sizer => 'X'),
+      'global'); }
+  return $token; }
+
+#======================================================================
+
 sub pushFrame {
   my ($self, $nobox) = @_;
   # Easy: just push a new undo hash.
@@ -492,7 +538,7 @@ sub pushFrame {
 sub popFrame {
   my ($self) = @_;
   if ($$self{undo}[0]{_FRAME_LOCK_}) {
-    Fatal('unexpected', '<endgroup>', $self->getStomach,
+    Error('unexpected', '<endgroup>', $$self{stomach},
       "Attempt to pop last locked stack frame"); }
   else {
     my $undo = shift(@{ $$self{undo} });
@@ -516,19 +562,19 @@ sub getFrameDepth {
 sub beginSemiverbatim {
   my ($self, @extraspecials) = @_;
   # Is this a good/safe enough shorthand, or should we really be doing beginMode?
-  $self->pushFrame;
-  $self->assignValue(MODE    => 'text');
-  $self->assignValue(IN_MATH => 0);
-  map { $self->assignCatcode($_ => CC_OTHER, 'local') }
-    @{ $self->lookupValue('SPECIALS') }, @extraspecials;
-  $self->assignMathcode('\'' => 0x8000, 'local');
+  pushFrame($self);
+  assignValue($self, MODE    => 'text');
+  assignValue($self, IN_MATH => 0);
+  map { assignCatcode($self, $_ => CC_OTHER, 'local') }
+    @{ lookupValue($self, 'SPECIALS') }, @extraspecials;
+  assignMathcode($self, '\'' => 0x8000, 'local');
   # try to stay as ASCII as possible
-  $self->assignValue(font => $self->lookupValue('font')->merge(encoding => 'ASCII'), 'local');
+  assignValue($self, font => lookupValue($self, 'font')->merge(encoding => 'ASCII'), 'local');
   return; }
 
 sub endSemiverbatim {
   my ($self) = @_;
-  $self->popFrame;
+  popFrame($self);
   return; }
 
 #======================================================================
@@ -547,11 +593,11 @@ sub pushDaemonFrame {
         my $type  = ref $value;
         if (($type eq 'HASH') || ($type eq 'ARRAY')) {    # Only concerned with mutable perl data?
                                                           # Local assignment
-          $$frame{$table}{$key} = 1;                      # Note new value in this frame.
+          $$frame{$table}{$key} = 1;                                  # Note new value in this frame.
           unshift(@{ $$hash{$key} }, daemon_copy($value)); } } } }    # And push new binding.
       # Record the contents of LaTeXML::Package::Pool as preloaded
   my $pool_preloaded_hash = { map { $_ => 1 } keys %LaTeXML::Package::Pool:: };
-  $self->assignValue('_PRELOADED_POOL_', $pool_preloaded_hash, 'global');
+  assignValue($self, '_PRELOADED_POOL_', $pool_preloaded_hash, 'global');
   # Now mark the top frame as LOCKED!!!
   $$frame{_FRAME_LOCK_} = 1;
   return; }
@@ -569,22 +615,22 @@ sub daemon_copy {
 sub popDaemonFrame {
   my ($self) = @_;
   while (!$$self{undo}[0]{_FRAME_LOCK_}) {
-    $self->popFrame; }
+    popFrame($self); }
   if (scalar(@{ $$self{undo} } > 1)) {
     delete $$self{undo}[0]{_FRAME_LOCK_};
     # Any non-preloaded Pool routines should be wiped away, as we
     # might want to reuse the Pool namespaces for the next run.
-    my $pool_preloaded_hash = $self->lookupValue('_PRELOADED_POOL_');
-    $self->assignValue('_PRELOADED_POOL_', undef, 'global');
+    my $pool_preloaded_hash = lookupValue($self, '_PRELOADED_POOL_');
+    assignValue($self, '_PRELOADED_POOL_', undef, 'global');
     foreach my $subname (keys %LaTeXML::Package::Pool::) {
       unless (exists $$pool_preloaded_hash{$subname}) {
         undef $LaTeXML::Package::Pool::{$subname};
         delete $LaTeXML::Package::Pool::{$subname};
     } }
     # Finally, pop the frame
-    $self->popFrame; }
+    popFrame($self); }
   else {
-    Fatal('unexpected', '<endgroup>', $self->getStomach,
+    Fatal('unexpected', '<endgroup>', $$self{stomach},
       "Daemon Attempt to pop last stack frame"); }
   return; }
 
@@ -617,7 +663,7 @@ sub activateScope {
         # Here we ALWAYS push the stashed values into the table
         # since they may be popped off by deactivateScope
         my ($table, $key, $value) = @$entry;
-        $$frame{$table}{$key}++;    # Note that this many values must be undone
+        $$frame{$table}{$key}++;                             # Note that this many values must be undone
         unshift(@{ $$self{$table}{$key} }, $value); } } }    # And push new binding.
   return; }
 
@@ -638,7 +684,7 @@ sub deactivateScope {
           shift(@{ $$self{$table}{$key} });
           $$frame{$table}{$key}--; }
         else {
-          Warn('internal', $key, $self->getStomach,
+          Warn('internal', $key, $$self{stomach},
             "Unassigning wrong value for $key from table $table in deactivateScope",
             "value is $value but stack is " . join(', ', @{ $$self{$table}{$key} })); } } } }
   return; }
@@ -661,11 +707,11 @@ sub convertUnit {
   $unit = lc($unit);
   # Put here since it could concievably evolve to depend on the current font.
   # Eventually try to track font size?
-  if    ($unit eq 'em') { return 10.0 * 65536; }
-  elsif ($unit eq 'ex') { return 4.3 * 65536; }
-  elsif ($unit eq 'mu') { return 10.0 * 65536 / 18; }
+  if    ($unit eq 'em') { return lookupValue($self, 'font')->getEMWidth; }
+  elsif ($unit eq 'ex') { return lookupValue($self, 'font')->getEXHeight; }
+  elsif ($unit eq 'mu') { return lookupValue($self, 'font')->getMUWidth; }
   else {
-    my $units = $self->lookupValue('UNITS');
+    my $units = lookupValue($self, 'UNITS');
     my $sp    = $$units{$unit};
     if (!$sp) {
       Warn('expected', '<unit>', undef, "Illegal unit of measure '$unit', assuming pt.");
@@ -688,26 +734,42 @@ sub getStatus {
   my ($self, $type) = @_;
   return $$self{status}{$type}; }
 
+our $SUCCESS_MESSAGE = 'No obvious problems';
+
 sub getStatusMessage {
   my ($self) = @_;
   my $status = $$self{status};
   my @report = ();
-  push(@report, colorizeString("$$status{warning} warning" . ($$status{warning} > 1 ? 's' : ''), 'warning'))
-    if $$status{warning};
-  push(@report, colorizeString("$$status{error} error" . ($$status{error} > 1 ? 's' : ''), 'error'))
-    if $$status{error};
-  push(@report, "$$status{fatal} fatal error" . ($$status{fatal} > 1 ? 's' : ''))
+  my $warning_status = $$status{warning} && colorizeString("$$status{warning} warning" . ($$status{warning} > 1 ? 's' : ''), 'warning');
+  my $error_status = $$status{error} && colorizeString("$$status{error} error" . ($$status{error} > 1 ? 's' : ''), 'error');
+  my $fatal_status = $$status{fatal} && colorizeString("$$status{fatal} fatal error" . ($$status{fatal} > 1 ? 's' : ''), 'fatal');
+  my @undef        = ($$status{undefined} ? keys %{ $$status{undefined} } : ());
+  my $undef_status = @undef && colorizeString(scalar(@undef) . " undefined macro" . (@undef > 1 ? 's' : '')
+      . "[" . join(', ', @undef) . "]", 'details');
+  my @miss           = ($$status{missing} ? keys %{ $$status{missing} } : ());
+  my $missing_status = @miss && colorizeString(scalar(@miss) . " missing file" . (@miss > 1 ? 's' : '')
+      . "[" . join(', ', @miss) . "]", 'details');
 
-    if $$status{fatal};
-  my @undef = ($$status{undefined} ? keys %{ $$status{undefined} } : ());
-  push(@report, colorizeString(scalar(@undef) . " undefined macro" . (@undef > 1 ? 's' : '')
-        . "[" . join(', ', @undef) . "]", 'details'))
-    if @undef;
-  my @miss = ($$status{missing} ? keys %{ $$status{missing} } : ());
-  push(@report, colorizeString(scalar(@miss) . " missing file" . (@miss > 1 ? 's' : '')
-        . "[" . join(', ', @miss) . "]", 'details'))
-    if @miss;
-  return join('; ', @report) || colorizeString('No obvious problems', 'success'); }
+  my $success_status = $SUCCESS_MESSAGE;
+  if ($LaTeXML::Common::Error::IS_TERMINAL) {
+    $warning_status = $warning_status && colorizeString($warning_status, 'warning');
+    $error_status   = $error_status   && colorizeString($error_status,   'error');
+    $fatal_status   = $fatal_status   && colorizeString($fatal_status,   'fatal');
+    $undef_status   = $undef_status   && colorizeString($undef_status,   'details');
+    $missing_status = $missing_status && colorizeString($missing_status, 'details');
+    $success_status = colorizeString($success_status, 'success'); }
+
+  push(@report, $warning_status) if $warning_status;
+  push(@report, $error_status)   if $error_status;
+  push(@report, $fatal_status)   if $fatal_status;
+  push(@report, $undef_status)   if $undef_status;
+  push(@report, $missing_status) if $missing_status;
+
+  my $message = join('; ', @report) || $success_status;
+  $message .= " (See $LaTeXML::Common::Error::LOG_PATH)"
+    if ($$status{fatal} || $$status{error} || $$status{warning})
+    && $LaTeXML::Common::Error::LOG && !ref $LaTeXML::Common::Error::LOG_PATH;
+  return $message; }
 
 sub getStatusCode {
   my ($self) = @_;
@@ -737,7 +799,7 @@ C<LaTeXML::Core::State> - stores the current state of processing.
 
 A C<LaTeXML::Core::State> object stores the current state of processing.
 It recording catcodes, variables values, definitions and so forth,
-as well as mimicing TeX's scoping rules.
+as well as mimicking TeX's scoping rules.
 
 =head2 Access to State and Processing
 
@@ -756,7 +818,7 @@ Returns the current Model representing the document model.
 =head2 Scoping
 
 The assignment methods, described below, generally take a C<$scope> argument, which
-determines how the assignment is made.  The allowed values and thier implications are:
+determines how the assignment is made.  The allowed values and their implications are:
 
  global   : global assignment.
  local    : local assignment, within the current grouping.
@@ -804,10 +866,8 @@ Assign $value to be associated with the the string C<$name>, according
 to the given scoping rule.
 
 Values are also used to specify most configuration parameters (which can
-therefor also be scoped).  The recognized configuration parameters are:
+therefore also be scoped).  The recognized configuration parameters are:
 
- VERBOSITY         : the level of verbosity for debugging
-                     output, with 0 being default.
  STRICT            : whether errors (eg. undefined macros)
                      are fatal.
  INCLUDE_COMMENTS  : whether to preserve comments in the
