@@ -27,7 +27,8 @@ use LaTeXML::Common::Font;
 # Silly place to import these....?
 use LaTeXML::Common::Color;
 use LaTeXML::Core::Definition;
-use base qw(LaTeXML::Common::Object);
+use Scalar::Util qw(blessed);
+use base         qw(LaTeXML::Common::Object);
 
 #**********************************************************************
 sub new {
@@ -41,6 +42,7 @@ sub initialize {
   my ($self) = @_;
   $$self{boxing}      = [];
   $$self{token_stack} = [];
+  $$self{progress}    = 0;
   $STATE->assignValue(MODE              => 'text',           'global');
   $STATE->assignValue(IN_MATH           => 0,                'global');
   $STATE->assignValue(PRESERVE_NEWLINES => 1,                'global');
@@ -52,6 +54,8 @@ sub initialize {
   $STATE->assignValue(mathfont => LaTeXML::Common::Font->mathDefault(), 'global');
   return; }
 
+our $DIGESTION_PROGRESS_QUANTUM = 30000;
+
 #**********************************************************************
 sub getGullet {
   my ($self) = @_;
@@ -85,18 +89,31 @@ sub getScriptLevel {
 sub digestNextBody {
   my ($self, $terminal) = @_;
   no warnings 'recursion';
-  my $startloc  = $self->getLocator;
+  my $startloc  = getLocator($self);
   my $initdepth = scalar(@{ $$self{boxing} });
   my $token;
   local @LaTeXML::LIST = ();
-  while (defined($token = $$self{gullet}->readXToken(1, 1))) {    # Done if we run out of tokens
-    push(@LaTeXML::LIST, $self->invokeToken($token));
+  my $alignment = $STATE->lookupValue('Alignment');
+  my @aug       = ();
+
+  while (defined($token = $$self{gullet}->getPendingComment || $$self{gullet}->readXToken(1))) {
+    if ($alignment && scalar(@LaTeXML::LIST) && (Equals($token, T_ALIGN) ||
+        Equals($token, T_CS('\cr')) || Equals($token, T_CS('\hidden@cr')) ||
+        Equals($token, T_CS('\hidden@crcr')))) {
+      # at least \over calls in here without the intent to passing through the alignment.
+      # So if we already have some digested boxes available, return them here.
+      $$self{gullet}->unread($token);
+      return @LaTeXML::LIST; }
+    my @r = invokeToken($self, $token);
+    push(@LaTeXML::LIST, @r);
+    push(@aug, $token, @r);
     last if $terminal and Equals($token, $terminal);
-    last if $initdepth > scalar(@{ $$self{boxing} }); }           # if we've closed the initial mode.
+    last if $initdepth > scalar(@{ $$self{boxing} }); }    # if we've closed the initial mode.
   Warn('expected', $terminal, $self, "body should have ended with '" . ToString($terminal) . "'",
-    "current body started at " . ToString($startloc))
+    "current body started at " . ToString($startloc),
+    "Got " . join("\n -- ", map { Stringify($_) } @aug))
     if $terminal && !Equals($token, $terminal);
-  push(@LaTeXML::LIST, Box()) unless $token;                      # Dummy `trailer' if none explicit.
+  push(@LaTeXML::LIST, Box()) unless $token;               # Dummy `trailer' if none explicit.
   return @LaTeXML::LIST; }
 
 # Digest a list of tokens independent from any current Gullet.
@@ -110,21 +127,14 @@ sub digest {
     $$self{gullet}->readingFromMouth(LaTeXML::Core::Mouth->new(), sub {
       my ($gullet) = @_;
       $gullet->unread($tokens);
-      $STATE->clearPrefixes;                                             # prefixes shouldn't apply here.
+      $STATE->clearPrefixes;    # prefixes shouldn't apply here.
       my $ismath    = $STATE->lookupValue('IN_MATH');
       my $initdepth = scalar(@{ $$self{boxing} });
-      my $depth     = $initdepth;
       local @LaTeXML::LIST = ();
-      while (defined(my $token = $$self{gullet}->readXToken(1, 1))) {    # Done if we run out of tokens
-        push(@LaTeXML::LIST, $self->invokeToken($token));
-        last if $initdepth > scalar(@{ $$self{boxing} }); }              # if we've closed the initial mode.
-      Fatal('internal', '<EOF>', $self,
-        "We've fallen off the end, somehow!?!?!",
-        "Last token " . ToString($LaTeXML::CURRENT_TOKEN)
-          . " (Boxing depth was $initdepth, now $depth: Boxing generated by "
-          . join(', ', map { ToString($_) } @{ $$self{boxing} }))
-        if $initdepth < $depth;
-
+      while (defined(my $token =
+            $$self{gullet}->getPendingComment || $$self{gullet}->readXToken(1))) {
+        push(@LaTeXML::LIST, invokeToken($self, $token));
+        last if $initdepth > scalar(@{ $$self{boxing} }); }    # if we've closed the initial mode.
       List(@LaTeXML::LIST, mode => ($ismath ? 'math' : 'text'));
     }); }
 
@@ -144,9 +154,10 @@ our @CATCODE_ABSORBABLE = (    # [CONSTANT]
   0, 0, 0, 0);
 
 sub invokeToken {
-  my ($self, $token) = @_;
   no warnings 'recursion';
+  my ($self, $token) = @_;
 INVOKE:
+  ProgressStep() if ($$self{progress}++ % $DIGESTION_PROGRESS_QUANTUM) == 0;
   push(@{ $$self{token_stack} }, $token);
   if (scalar(@{ $$self{token_stack} }) > $MAXSTACK) {
     Fatal('internal', '<recursion>', $self,
@@ -157,22 +168,29 @@ INVOKE:
   my $meaning = $STATE->lookupDigestableDefinition($token);
 
   if (!$meaning) {
-    @result = $self->invokeToken_undefined($token); }
+    @result = invokeToken_undefined($self, $token); }
   elsif ($meaning->isaToken) {    # Common case
     my $cc = $meaning->getCatcode;
     if ($cc == CC_CS) {
-      @result = $self->invokeToken_undefined($token); }
+      @result = invokeToken_undefined($self, $token); }
     elsif ($CATCODE_ABSORBABLE[$cc]) {
-      @result = $self->invokeToken_simple($token, $meaning); }
+      @result = invokeToken_simple($self, $token, $meaning); }
     else {
+      # Special error guard for the align char "&":
+      # Locally deactivate to avoid a flurry of errors in the same table.
+      # Alert the end user once per table, allowing longer documents to not
+      # hit 100 errors too quickly.
+      $STATE->assignMeaning($token,
+        $STATE->lookupMeaning(T_CS('\relax')), 'local')
+        if Equals($token, T_ALIGN);
       Error('misdefined', $token, $self,
         "The token " . Stringify($token) . " should never reach Stomach!");
-      @result = $self->invokeToken_simple($token, $meaning); } }
+      @result = invokeToken_simple($self, $token, $meaning); } }
   # A math-active character will (typically) be a macro,
   # but it isn't expanded in the gullet, but later when digesting, in math mode (? I think)
   elsif ($meaning->isExpandable) {
     my $gullet = $$self{gullet};
-    $gullet->unread(@{ $meaning->invoke($gullet) || [] });
+    $gullet->unread($meaning->invoke($gullet));
     $token = $gullet->readXToken();    # replace the token by it's expansion!!!
     pop(@{ $$self{token_stack} });
     goto INVOKE if $token; }
@@ -180,35 +198,42 @@ INVOKE:
     @result = $meaning->invoke($self);
     $STATE->clearPrefixes unless $meaning->isPrefix; }    # Clear prefixes unless we just set one.
   else {
-    Fatal('misdefined', $meaning, $self,
-      "The object " . Stringify($meaning) . " should never reach Stomach!"); }
+    Error('misdefined', $meaning, $self,
+      "The object " . Stringify($meaning) . " should never reach Stomach!");
+    @result = (makeMisdefinedError($meaning)); }
   if ((scalar(@result) == 1) && (!defined $result[0])) {
     @result = (); }                                       # Just paper over the obvious thing.
-  Fatal('misdefined', $token, $self,
-    "Execution yielded non boxes",
-    "Returned " . join(',', map { "'" . Stringify($_) . "'" }
-        grep { (!ref $_) || (!$_->isaBox) } @result))
-    if grep { (!ref $_) || (!$_->isaBox) } @result;
-
+  if (my @bad = grep { (!blessed $_) || (!$_->isaBox) } @result) {
+    Error('misdefined', $token, $self,
+      "Execution yielded non boxes",
+      "Returned " . join(',', map { "'" . Stringify($_) . "'" } @bad));
+    @result = (makeMisdefinedError(@result)); }
   pop(@{ $$self{token_stack} });
   return @result; }
 
+sub makeMisdefinedError {
+  my (@objects) = @_;
+  return LaTeXML::Core::Whatsit->new($STATE->lookupDefinition(T_CS('\@ERROR')),
+    ['misdefined', join('', map { ToString($_); } @objects)],
+    font => $STATE->lookupValue('font'),
+  ); }
+
 sub invokeToken_undefined {
   my ($self, $token) = @_;
   $STATE->generateErrorStub($self, $token);
-  $self->getGullet->unread($token);    # Retry
+  $$self{gullet}->unread($token);    # Retry
   return; }
 
 sub invokeToken_simple {
   my ($self, $token, $meaning) = @_;
   my $cc   = $meaning->getCatcode;
   my $font = $STATE->lookupValue('font');
-  $STATE->clearPrefixes;               # prefixes shouldn't apply here.
   if ($cc == CC_SPACE) {
+    $STATE->clearPrefixes;                   # prefixes shouldn't apply here.
     if ($STATE->lookupValue('IN_MATH')) {    # (but in Preamble, OK ?)
       return (); }
     else {
-      return Box($meaning->toString, $font, $self->getGullet->getLocator, $meaning); } }
+      return Box($meaning->toString, $font, $$self{gullet}->getLocator, $meaning); } }
   elsif ($cc == CC_COMMENT) {                # Note: Comments need char decoding as well!
     my $comment = LaTeXML::Package::FontDecodeString($meaning->toString, undef, 1);
     # However, spaces normally would have be digested away as positioning...
@@ -216,6 +241,7 @@ sub invokeToken_simple {
     $comment =~ s/\Q$badspace\E/ /g;
     return LaTeXML::Core::Comment->new($comment); }
   else {
+    $STATE->clearPrefixes;                          # prefixes shouldn't apply here.
     return Box(LaTeXML::Package::FontDecodeString($meaning->toString, undef, 1),
       undef, undef, $meaning); } }
 
@@ -243,7 +269,7 @@ sub pushStackFrame {
   $STATE->assignValue(afterAssignment       => undef,                   'local');  # ALWAYS bind this!
   $STATE->assignValue(groupNonBoxing        => $nobox,                  'local');  # ALWAYS bind this!
   $STATE->assignValue(groupInitiator        => $LaTeXML::CURRENT_TOKEN, 'local');
-  $STATE->assignValue(groupInitiatorLocator => $self->getLocator,       'local');
+  $STATE->assignValue(groupInitiatorLocator => getLocator($self),       'local');
   push(@{ $$self{boxing} }, $LaTeXML::CURRENT_TOKEN) unless $nobox;    # For begingroup/endgroup
   return; }
 
@@ -253,7 +279,8 @@ sub popStackFrame {
     if (@$beforeafter) {
       my @result = map { $_->beDigested($self) } @$beforeafter;
       if (my ($x) = grep { !$_->isaBox } @result) {
-        Fatal('misdefined', $x, $self, "Expected a Box|List|Whatsit, but got '" . Stringify($x) . "'"); }
+        Error('misdefined', $x, $self, "Expected a Box|List|Whatsit, but got '" . Stringify($x) . "'");
+        @result = (makeMisdefinedError(@result)); }
       push(@LaTeXML::LIST, @result); } }
   my $after = $STATE->lookupValue('afterGroup');
   $STATE->popFrame;
@@ -292,7 +319,7 @@ sub egroup {
   if (    ##$STATE->isValueBound('MODE', 0) ||    # Last stack frame was a mode switch!?!?!
     $STATE->lookupValue('groupNonBoxing')) {    # or group was opened with \begingroup
     Error('unexpected', $LaTeXML::CURRENT_TOKEN, $self, "Attempt to close boxing group",
-      $self->currentFrameMessage); }
+      currentFrameMessage($self)); }
   else {                                        # Don't pop if there's an error; maybe we'll recover?
     popStackFrame($self, 0); }
   return; }
@@ -307,7 +334,7 @@ sub endgroup {
   if (    ##$STATE->isValueBound('MODE', 0) ||    # Last stack frame was a mode switch!?!?!
     !$STATE->lookupValue('groupNonBoxing')) {    # or group was opened with \bgroup
     Error('unexpected', $LaTeXML::CURRENT_TOKEN, $self, "Attempt to close non-boxing group",
-      $self->currentFrameMessage); }
+      currentFrameMessage($self)); }
   else {                                         # Don't pop if there's an error; maybe we'll recover?
     popStackFrame($self, 1); }
   return; }
@@ -341,13 +368,13 @@ sub setMode {
     # but inherit color and size
     $STATE->assignValue(font => $STATE->lookupValue('savedfont')->merge(
         color => $curfont->getColor, background => $curfont->getBackground,
-        size => $curfont->getSize), 'local'); }
+        size  => $curfont->getSize), 'local'); }
   return; }
 
 sub beginMode {
   my ($self, $mode) = @_;
-  $self->pushStackFrame;    # Effectively bgroup
-  $self->setMode($mode);
+  pushStackFrame($self);    # Effectively bgroup
+  setMode($self, $mode);
   return; }
 
 sub endMode {
@@ -355,9 +382,9 @@ sub endMode {
   if ((!$STATE->isValueBound('MODE', 0))    # Last stack frame was NOT a mode switch!?!?!
     || ($STATE->lookupValue('MODE') ne $mode)) {    # Or was a mode switch to a different mode
     Error('unexpected', $LaTeXML::CURRENT_TOKEN, $self, "Attempt to end mode $mode",
-      $self->currentFrameMessage); }
+      currentFrameMessage($self)); }
   else {    # Don't pop if there's an error; maybe we'll recover?
-    $self->popStackFrame; }    # Effectively egroup.
+    popStackFrame($self); }    # Effectively egroup.
   return; }
 
 #**********************************************************************
