@@ -26,13 +26,10 @@ our @EXPORT = (@LaTeXML::Common::Error::EXPORT);
 sub new {
   my ($class, %options) = @_;
   my $self = bless { status => {}, %options }, $class;
-  $$self{verbosity} = 0 unless defined $$self{verbosity};
   # TEMPORARY HACK!!!!
-  # Create a State object, essentially only to hold verbosity (for now)
-  # so that Errors can be reported, managed and recorded
-  # Eventually will be a "real" State (or other configuration object)
+  # Create a State object, for config & recursive conversions;
+  # Eventually should be managed higher up.
   $$self{state} = LaTeXML::Core::State->new();
-  $$self{state}->assignValue(VERBOSITY => $$self{verbosity});
   return $self; }
 
 #======================================================================
@@ -48,7 +45,7 @@ sub ProcessChain_internal {
   local $LaTeXML::Post::DOCUMENT = $doc;
 
   my @docs = ($doc);
-  NoteBegin("post-processing");
+  ProgressSpinup("post-processing");
 
   foreach my $processor (@postprocessors) {
     local $LaTeXML::Post::PROCESSOR = $processor;
@@ -56,17 +53,17 @@ sub ProcessChain_internal {
     foreach my $doc (@docs) {
       local $LaTeXML::Post::DOCUMENT = $doc;
       if (my @nodes = grep { $_ } $processor->toProcess($doc)) {    # If there are nodes to process
-        my $n = scalar(@nodes);
+        my $n   = scalar(@nodes);
         my $msg = join(' ', $processor->getName || '',
           $doc->siteRelativeDestination || '',
           ($n > 1 ? "$n to process" : 'processing'));
-        NoteBegin($msg);
+        ProgressSpinup($msg);
         push(@newdocs, $processor->process($doc, @nodes));
-        NoteEnd($msg); }
+        ProgressSpindown($msg); }
       else {
         push(@newdocs, $doc); } }
     @docs = @newdocs; }
-  NoteEnd("post-processing");
+  ProgressSpindown("post-processing");
   return @docs; }
 
 ## HACK!!!
@@ -106,7 +103,7 @@ sub getStatusMessage {
 sub getsorter {
   my ($self, $lang) = @_;
   my $collator;
-  if ($collator = $$self{collatorcache}{$lang}) { }
+  if    ($collator = $$self{collatorcache}{$lang}) { }
   elsif ($collator = eval {
       local $LaTeXML::IGNORE_ERRORS = 1;
       require 'Unicode/Collate/Locale.pm';
@@ -157,7 +154,6 @@ use base qw(LaTeXML::Common::Object);
 sub new {
   my ($class, %options) = @_;
   my $self = bless {%options}, $class;
-  $$self{verbosity}          = 0 unless defined $$self{verbosity};
   $$self{resource_directory} = $options{resource_directory};
   $$self{resource_prefix}    = $options{resource_prefix};
   my $name = $class; $name =~ s/^LaTeXML::Post:://;
@@ -195,17 +191,77 @@ sub desiredResourcePathname {
 # but I think we've accommodated absolute ones.
 sub generateResourcePathname {
   my ($self, $doc, $node, $source, $type) = @_;
-  my $subdir = $$self{resource_directory} || '';
-  my $prefix = $$self{resource_prefix}    || "x";
+  my $subdir  = $$self{resource_directory} || '';
+  my $prefix  = $$self{resource_prefix}    || "x";
   my $counter = join('_', "_max", $subdir, $prefix, "counter_");
-  my $n = $doc->cacheLookup($counter) || 0;
-  my $name = $prefix . ++$n;
+  my $n       = $doc->cacheLookup($counter) || 0;
+  my $name    = $prefix . ++$n;
   $doc->cacheStore($counter, $n);
   return pathname_make(dir => $subdir, name => $name, type => $type); }
 
+# Returns a two-part list of the form:
+#
+# [class, classoptions, oldstyle], [package1,package1options], [package2,package2options], ...
+#
+# Where there first element is always the *class* (if none, "article" returned as a default)
+# And all following elements are *packages*
+#
+sub find_documentclass_and_packages {
+  my ($self, $doc) = @_;
+  my ($class, $classoptions, $oldstyle, @packages);
+  foreach my $pi ($doc->findnodes(".//processing-instruction('latexml')")) {
+    my $data  = $pi->textContent;
+    my $entry = {};
+    while ($data =~ s/\s*([\w\-\_]*)=([\"\'])(.*?)\2//) {
+      $$entry{$1} = $3; }
+    if ($$entry{class}) {
+      $class        = $$entry{class};
+      $classoptions = $$entry{options} || 'onecolumn';
+      $oldstyle     = $$entry{oldstyle}; }
+    elsif ($$entry{package}) {
+      my @p = grep { $_; } split(/\s*,\s*/, $$entry{package});
+      foreach my $package (@p) {
+        push(@packages, [$package, $$entry{options} || '']); } } }
+  if (!$class) {
+    Warn('expected', 'class', undef, "No document class found; using article");
+    $class = 'article'; }
+  return ([$class, $classoptions, $oldstyle], @packages); }
+
+sub find_preambles {
+  my ($self, $doc) = @_;
+  my @preambles = ();
+  foreach my $pi ($doc->findnodes(".//processing-instruction('latexml')")) {
+    my $data = $pi->textContent;
+    while ($data =~ s/\s*([\w\-\_]*)=([\"\'])(.*?)\2//) {
+      if ($1 eq 'preamble') {
+        push(@preambles, $3); } } }
+  return join("\n", @preambles); }
+
+# Utility to Copy any foreign attributes from $node onto the $newnode
+# Allow both $node & $newnode to be XML Elements ore array representation
+sub copy_foreign_attributes {
+  my ($self, $newnode, $node) = @_;
+  my %attr = ();
+  if (ref $node eq 'ARRAY') {
+    %attr = %{ $$node[1] }; }
+  else {
+    foreach my $attr ($node->attributes()) {
+      if ($attr->nodeType == XML_ATTRIBUTE_NODE) {
+        $attr{ $attr->nodeName } = $attr->getValue; } } }
+  foreach my $key (keys %attr) {
+    next if $key     =~ /^xml:/;
+    next unless $key =~ /:/;
+    my $value = $attr{$key};
+    if (ref $newnode eq 'ARRAY') {
+      $$newnode[1]{$key} = $value unless defined $$newnode[1]{$key}; }
+    else {
+      $newnode->setAttribute($key => $value) unless $newnode->hasAttribute($key); } }
+  return; }
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 package LaTeXML::Post::MathProcessor;
 use strict;
+use LaTeXML::Util::Pathname;
 use LaTeXML::Post;
 use LaTeXML::Common::Error;
 use base qw(LaTeXML::Post::Processor);
@@ -281,9 +337,11 @@ sub process {
       # Now do cross referencing
       $proc1->addCrossrefs($doc, $proc2);
       $proc2->addCrossrefs($doc, $proc1); } }
-  NoteProgressDetailed(" [converted $n Maths]");
+  NoteLog("converted $n Maths");
   return $doc; }
 
+sub canConvert { return 1; }
+
 # Make THIS MathProcessor the primary branch (of whatever parallel markup it supports),
 # and make all of the @moreprocessors be secondary ones.
 sub setParallel {
@@ -314,9 +372,11 @@ sub processNode {
   # XMath will be removed (LATER!), but mark its ids as reusable.
   $doc->preremoveNodes($xmath);
   if ($$self{parallel}) {
-    my $primary = $self->convertNode($doc, $xmath);
+    my $primary     = $self->convertNode($doc, $xmath);
     my @secondaries = ();
     foreach my $proc (@{ $$self{secondary_processors} }) {
+      # Exception: Do not generate Content MathML for kludge parses
+      next unless $proc->canConvert($doc, $math);
       local $LaTeXML::Post::MATHPROCESSOR = $proc;
       my $secondary = $proc->convertNode($doc, $xmath);
       # IF it is (first) image, copy image attributes to ltx:Math ???
@@ -330,6 +390,9 @@ sub processNode {
   # (if there's an XMath PostProcessing module, it will add it back, with appropriate id's)
   if (my $xml = $$conversion{xml}) {
     $$conversion{xml} = $self->outerWrapper($doc, $xmath, $xml); }
+  elsif (my $string = $$conversion{string}) {
+    my $mimetype = $$conversion{mimetype} || 'unknown';
+    $$conversion{xml} = ['ltx:text', { class => 'ltx_math_' . $mimetype }, $string]; }
   $doc->removeNodes($xmath);
   # NOTE: Unless XMath is the primary, (preserving the XMath, w/no IDSuffix)
   # we've got to remove the id's from the XMath, since the primary will get same id's
@@ -348,7 +411,7 @@ sub maybeSetMathImage {
   if ((($$conversion{mimetype} || '') =~ /^image\//)    # Got an image?
     && !$math->getAttribute('imagesrc')) {              # and it's the first one
     if (my $src = $$conversion{src}) {
-      $math->setAttribute(imagesrc    => $src);
+      $math->setAttribute(imagesrc    => pathname_to_url($src));
       $math->setAttribute(imagewidth  => $$conversion{width});
       $math->setAttribute(imageheight => $$conversion{height});
       $math->setAttribute(imagedepth  => $$conversion{depth}); } }
@@ -375,7 +438,7 @@ sub convertNode {
 # Maybe the caller of this should check the namespaces, and call wrapper if needed?
 sub combineParallel {
   my ($self, $doc, $xmath, $primary, @secondaries) = @_;
-  LaTeXML::Post::Error('misdefined', (ref $self), undef,
+  Error('misdefined', (ref $self), undef,
     "Abstract package: combining parallel markup has not been defined for this MathProcessor",
     "dropping the extra markup from: " . join(',', map { $$_{processor} } @secondaries));
   return $primary; }
@@ -403,7 +466,7 @@ sub convertXMTextContent {
       my $tag = $doc->getQName($node);
       if ($tag eq 'ltx:XMath') {
         my $conversion = $self->convertNode($doc, $node);
-        my $xml = $$conversion{xml};
+        my $xml        = $$conversion{xml};
         # And if no xml ????
         push(@result, $self->outerWrapper($doc, $node, $xml)); }
       else {
@@ -443,16 +506,20 @@ sub rawIDSuffix {
 # * the containing XMDual (which makes more sense when we've generated a "container")
 # * the containing XMDual's semantic operator (makes more sense when we're generating
 #   tokens that are only visible from the presentation branch)
+#
+# Additionally: piggy-back here copying any foreign-namespaced attributes from source to target
 sub associateNode {
   my ($self, $node, $currentnode, $noxref) = @_;
   my $r = ref $node;
   return unless $currentnode && $r && ($r eq 'ARRAY' || $r eq 'XML::LibXML::Element');
+  $self->copy_foreign_attributes($node, $currentnode);
   my $document = $LaTeXML::Post::DOCUMENT;
   # Check if already associated with a source node
   my $isarray = ref $node eq 'ARRAY';
   # What kind of branch are we generating?
   my $ispresentation = $self->rawIDSuffix eq '.pmml';    # TEMPORARY HACK: BAAAAD method!
   my $iscontainer    = 0;
+  my $container;
   if ($isarray) {
     return if $$node[1]{'_sourced'};
     $$node[1]{'_sourced'} = 1;
@@ -463,22 +530,38 @@ sub associateNode {
     $node->setAttribute('_sourced' => 1);
     $iscontainer = scalar(element_nodes($node)); }
   my $sourcenode = $currentnode;
-  # If the generated node is a "container" (non-token!), use the container as source
-  if ($iscontainer) {
-    if (my $container = $document->findnode('ancestor-or-self::ltx:XMDual[1]', $sourcenode)) {
+  # If the current node is declared, use it (but meaning is overridden)
+  if ($currentnode->getAttribute('decl_id')) { }
+  # If the generated node is a "container" (non-token!), use the containing XMDual as source
+  elsif ($iscontainer) {
+    my $sid = $sourcenode->getAttribute('xml:id');
+    # But ONLY if that XMDual is the "direct" parent, or is parent of XRef that points to $current
+    if ($container = $document->findnode('parent::ltx:XMDual[1]', $sourcenode)
+      || ($sid &&
+        $document->findnode("ancestor-or-self::ltx:XMDual[ltx:XMRef[\@idref='$sid']][1]",
+          $sourcenode))) {
       $sourcenode = $container; } }
+  # Parent App w/decl_id or meaning is source, unless current node has decl_id (but ignore meaning)
+  elsif ($container = $document->findnode('ancestor::ltx:XMApp[@decl_id or @meaning][1]', $sourcenode)) {
+    $sourcenode = $container; }
   # If the current node is appropriately visible, use it.
   elsif ($currentnode->getAttribute(($ispresentation ? '_cvis' : '_pvis'))) { }
   # Else (current node isn't visible); try to find content OPERATOR
-  elsif (my $container = $document->findnode('ancestor-or-self::ltx:XMDual[1]', $sourcenode)) {
+  elsif ($container = $document->findnode('ancestor-or-self::ltx:XMDual[1]', $sourcenode)) {
     my ($op) = element_nodes($container);
     my $q = $document->getQName($op) || 'unknown';
-    if ($q eq 'ltx:XMTok') { }
+    if ($container->hasAttribute('decl_id')) {
+      $op = undef; }
+    elsif ($q eq 'ltx:XMTok') { }
     elsif ($q eq 'ltx:XMApp') {
-      ($op) = element_nodes($op);
-      if ($document->getQName($op) eq 'ltx:XMRef') {
-        $op = $document->realizeXMNode($op); } }
-    if ($op && !$op->getAttribute('_pvis')) {
+      while (($q eq 'ltx:XMApp') && !$op->hasAttribute('meaning')) {
+        ($op) = element_nodes($op);
+        $q = $document->getQName($op) || 'unknown'; } }    # get "real" operator
+    if ($q eq 'ltx:XMRef') {
+      $op = $document->realizeXMNode($op); }
+    # Be a bit fuzzy about whether something is "visible"
+    if ($op && !($op->getAttribute('_pvis')
+        && (($op->getAttribute('thickness') || '<anything>') ne '0pt'))) {
       $sourcenode = $op; }
     else {
       $sourcenode = $container; } }
@@ -488,16 +571,16 @@ sub associateNode {
       $document->generateNodeID($sourcenode, '', 1); }         # but the ID is reusable
     if (my $sourceid = $sourcenode->getAttribute('fragid')) {    # If source has ID
       my $nodeid = $currentnode->getAttribute('fragid') || $sourceid;
-      my $id = $document->uniquifyID($nodeid, $self->IDSuffix);
+      my $id     = $document->uniquifyID($nodeid, $self->IDSuffix);
       if ($isarray) {
         $$node[1]{'xml:id'} = $id; }
       else {
         $node->setAttribute('xml:id' => $id); }
       push(@{ $$self{convertedIDs}{$sourceid} }, $id) unless $noxref; } }
   $self->associateNodeHook($node, $sourcenode, $noxref);
-  if ($isarray) {                                                # Array represented
+  if ($isarray) {    # Array represented
     map { $self->associateNode($_, $currentnode, $noxref) } @$node[2 .. $#$node]; }
-  else {                                                         # LibXML node
+  else {             # LibXML node
     map { $self->associateNode($_, $currentnode, $noxref) } element_nodes($node); }
   return; }
 
@@ -513,7 +596,7 @@ sub shownode {
   if ($ref eq 'ARRAY') {
     my ($tag, $attr, @children) = @$node;
     return "\n" . ('  ' x $level)
-      . '[' . $tag . ',{' . join(',', map { $_ . '=>' . $$attr{$_} } sort keys %$attr) . '},'
+      . '[' . $tag . ',{' . join(',', map { $_ . '=>' . ($$attr{$_} || '') } sort keys %$attr) . '},'
       . join(',', map { shownode($_, $level + 1) } @children) . ']'; }
   elsif ($ref =~ /^XML/) {
     return $node->toString; }
@@ -534,16 +617,42 @@ sub addCrossrefs {
   my $selfs_map  = $$self{convertedIDs};
   my $others_map = $$otherprocessor{convertedIDs};
   my $xrefids    = $$self{crossreferencing_ids};
-  foreach my $xid (keys %$selfs_map) {    # For each Math id that $self converted
-    if (my $other_ids = $$others_map{$xid}) {    # Did $other also convert those ids?
+  my $backref    = {};
+  foreach my $id (keys %$selfs_map) {
+    foreach my $t (@{ $$selfs_map{$id} }) {
+      $$backref{$t} = $id; } }
+  foreach my $xid (keys %$selfs_map) {    # For each XMath id that $self converted
+    my $other_ids = $$others_map{$xid};    # the ids where $xid ended up in $other processor
+    if (!$other_ids) {
+      # But If this node didn't directly end up in $other, try to find alternative
+      # Typically happens when a "visible" node doesn't have visible representation in other format!
+      # So, see if an ancestor got mapped.
+      if (my $mapped = $$selfs_map{$xid}) {
+        foreach my $mid (@$mapped) {
+          if (my $node = $doc->findNodeByID($mid)) {
+            my ($parent, $pid, $xpid) = ($node, undef, undef);
+            while (($parent = $parent->parentNode)
+              && ($parent->nodeType == XML_ELEMENT_NODE)      # in case we hit Document
+              && (!($pid = $parent->getAttribute('xml:id'))
+                || !($xpid = $$backref{$pid})
+                || !$$others_map{$xpid})) { }
+            if ($xpid) {
+              $other_ids = $$others_map{$xpid}; } } } } }
+    if ($other_ids) {    # Hopefully, we've got the targets, now
       my $xref_id = $$other_ids[0];
-      if (scalar(@$other_ids) > 1) {             # Find 1st in document order! (order is cached)
+      if (scalar(@$other_ids) > 1) {    # Find 1st in document order! (order is cached)
         ($xref_id) = sort { $$xrefids{$a} <=> $$xrefids{$b} } @$other_ids; }
       foreach my $id (@{ $$selfs_map{$xid} }) {    # look at each node $self created from $xid
         if (my $node = $doc->findNodeByID($id)) {    # If we find a node,
-          $self->addCrossref($node, $xref_id); } } } }    # add a crossref from it to $others's node
+          $self->addCrossref($node, $xref_id); } } }    # }    # add a crossref from it to $others's nod
+    else {
+  } }
   return; }
 
+sub mathIsParsed {
+  my ($doc, $math) = @_;
+  return $math && (($math->getAttribute('class') || '') !~ 'ltx_math_unparsed'); }
+
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 package LaTeXML::Post::Document;
@@ -553,8 +662,9 @@ use LaTeXML::Util::Pathname;
 use LaTeXML::Util::Radix;
 use DB_File;
 use Unicode::Normalize;
-use LaTeXML::Post;                                        # to import error handling...
+use LaTeXML::Post;    # to import error handling...
 use LaTeXML::Common::Error;
+use URI::file;
 use base qw(LaTeXML::Common::Object);
 our $NSURI = "http://dlmf.nist.gov/LaTeXML";
 our $XPATH = LaTeXML::Common::XML::XPath->new(ltx => $NSURI);
@@ -595,9 +705,9 @@ sub new_internal {
     else {
       $data{siteDirectory} = $data{destinationDirectory}; } }
   # Start, at least, with our own namespaces.
-  $data{namespaces}    = { ltx    => $NSURI } unless $data{namespaces};
-  $data{namespaceURIs} = { $NSURI => 'ltx' }  unless $data{namespaceURIs};
-  $data{idcache}       = {};
+  $data{namespaces}       = { ltx    => $NSURI } unless $data{namespaces};
+  $data{namespaceURIs}    = { $NSURI => 'ltx' }  unless $data{namespaceURIs};
+  $data{idcache}          = {};
   $data{idcache_reusable} = {};
   $data{idcache_reserve}  = {};
 
@@ -606,11 +716,18 @@ sub new_internal {
 
 sub newFromFile {
   my ($class, $source, %options) = @_;
-  $options{source} = $source;
+  my $path = (ref $class
+    ? pathname_find($source, paths => $$class{searchpaths})
+    : $source);
+  if (!$path) {
+    Error('missing_file', $source, $class, "No XML document '$source' found",
+      (ref $class ? "search paths are " . join(', ', @{ $$class{searchpaths} }) : ()));
+    return; }
+  $options{source} = $path;
   if (!$options{sourceDirectory}) {
-    my ($dir, $name, $ext) = pathname_split($source);
+    my ($dir, $name, $ext) = pathname_split($path);
     $options{sourceDirectory} = $dir || '.'; }
-  my $doc = $class->new(LaTeXML::Common::XML::Parser->new()->parseFile($source), %options);
+  my $doc = $class->new(LaTeXML::Common::XML::Parser->new()->parseFile($path), %options);
   $doc->validate if $$doc{validate};
   return $doc; }
 
@@ -686,11 +803,13 @@ sub setDocument_internal {
     foreach my $node ($self->findnodes("//*[\@xml:id]")) {    # Now record all ID's
       $$self{idcache}{ $node->getAttribute('xml:id') } = $node; }
     # Fetch any additional namespaces from the root
-    foreach my $ns ($root->documentElement->getNamespaces) {
-      my ($prefix, $uri) = ($ns->getLocalName, $ns->getData);
-      if ($prefix) {
-        $$self{namespaces}{$prefix} = $uri    unless $$self{namespaces}{$prefix};
-        $$self{namespaceURIs}{$uri} = $prefix unless $$self{namespaceURIs}{$uri}; } }
+
+    if (my $docroot = $root->documentElement) {
+      foreach my $ns ($docroot->getNamespaces) {
+        my ($prefix, $uri) = ($ns->getLocalName, $ns->getData);
+        if ($prefix) {
+          $$self{namespaces}{$prefix} = $uri    unless $$self{namespaces}{$prefix};
+          $$self{namespaceURIs}{$uri} = $prefix unless $$self{namespaceURIs}{$uri}; } } }
 
     # Extract data from latexml's ProcessingInstructions
     # I'd like to provide structured access to the PI's for those modules that need them,
@@ -703,8 +822,11 @@ sub setDocument_internal {
     foreach my $pi (@{ $$self{processingInstructions} }) {
       if ($pi =~ /^\s*searchpaths\s*=\s*([\"\'])(.*?)\1\s*$/) {
         push(@paths, split(',', $2)); } }
-    push(@paths, pathname_absolute($$self{sourceDirectory})) if $$self{sourceDirectory};
-    $$self{searchpaths} = [@paths]; }
+    ### No, this ultimately can be the xml source, which may be the destination;
+    ### adding this gets the wrong graphics (already processed!)
+    ### push(@paths, pathname_absolute($$self{sourceDirectory})) if $$self{sourceDirectory};
+    # But do add cwd after the document specified paths.
+    $$self{searchpaths} = [@paths, '.']; }
   elsif ($roottype eq 'XML::LibXML::Element') {
     $$self{document} = XML::LibXML::Document->new("1.0", "UTF-8");
     # Assume we've got any namespaces already ?
@@ -724,7 +846,7 @@ sub setDocument_internal {
     my ($tag, $attributes, @children) = @$root;
     my ($prefix, $localname) = $tag =~ /^(.*):(.*)$/;
     my $nsuri = $$self{namespaces}{$prefix};
-    my $node = $$self{document}->createElementNS($nsuri, $localname);
+    my $node  = $$self{document}->createElementNS($nsuri, $localname);
     $$self{document}->setDocumentElement($node);
     map { $$attributes{$_} && $node->setAttribute($_ => $$attributes{$_}) } keys %$attributes
       if $attributes;
@@ -734,6 +856,8 @@ sub setDocument_internal {
     $self->addNodes($node, @children); }
   else {
     Fatal('unexpected', $root, undef, "Dont know how to use '$root' as document element"); }
+  # set URI to destination
+  $$self{document}->setURI(URI::file->new($self->getDestination));
   return $self; }
 
 our @MonthNames = (qw( January February March April May June
@@ -745,12 +869,7 @@ sub addDate {
     my @dates;
     #  $fromdoc's document has some, so copy them.
     if ($fromdoc && (@dates = $fromdoc->findnodes('ltx:date', $fromdoc->getDocumentElement))) {
-      $self->addNodes($self->getDocumentElement, @dates); }
-    else {
-      my ($sec, $min, $hour, $mday, $mon, $year) = localtime(time());
-      $self->addNodes($self->getDocumentElement,
-        ['ltx:date', { role => 'creation' },
-          $MonthNames[$mon] . " " . $mday . ", " . (1900 + $year)]); } }
+      $self->addNodes($self->getDocumentElement, @dates); } }
   return; }
 
 #======================================================================
@@ -833,6 +952,10 @@ sub stringify {
   my ($self) = @_;
   return 'Post::Document[' . $self->siteRelativeDestination . ']'; }
 
+sub getLocator {
+  my ($self) = @_;
+  return $$self{source}; }
+
 #======================================================================
 sub validate {
   my ($self) = @_;
@@ -843,29 +966,31 @@ sub validate {
       $schema = $2; } }
   if ($schema) {    # Validate using rng
     my $rng = LaTeXML::Common::XML::RelaxNG->new($schema, searchpaths => [$self->getSearchPaths]);
-    LaTeXML::Post::Error('I/O', $schema, undef, "Failed to load RelaxNG schema $schema" . "Response was: $@")
+    Error('I/O', $schema, undef, "Failed to load RelaxNG schema $schema" . "Response was: $@")
       unless $rng;
     my $v = eval {
       local $LaTeXML::IGNORE_ERRORS = 1;
       $rng->validate($$self{document}); };
-    LaTeXML::Post::Error("malformed", 'document', undef,
+    Error("malformed", 'document', undef,
       "Document fails RelaxNG validation (" . $schema . ")",
-      "Validation reports: " . $@) if $@ || !defined $v; }
+      "Validation reports: " . $@,
+      "(Jing may provide a more precise report; https://relaxng.org/jclark/jing.html)")
+      if $@ || !defined $v; }
   elsif (my $decldtd = $$self{document}->internalSubset) {    # Else look for DTD Declaration
     my $dtd = XML::LibXML::Dtd->new($decldtd->publicId, $decldtd->systemId);
     if (!$dtd) {
-      LaTeXML::Post::Error("I/O", $decldtd->publicId, undef,
+      Error("I/O", $decldtd->publicId, undef,
         "Failed to load DTD " . $decldtd->publicId . " at " . $decldtd->systemId,
         "skipping validation"); }
     else {
       my $v = eval {
         local $LaTeXML::IGNORE_ERRORS = 1;
         $$self{document}->validate($dtd); };
-      LaTeXML::Post::Error("malformed", 'document', undef,
+      Error("malformed", 'document', undef,
         "Document failed DTD validation (" . $decldtd->systemId . ")",
         "Validation reports: " . $@) if $@ || !defined $v; } }
   else {    # Nothing found to validate with
-    LaTeXML::Post::Warn("expected", 'schema', undef,
+    Warn("expected", 'schema', undef,
       "No Schema or DTD found for this document"); }
   return; }
 
@@ -876,15 +1001,15 @@ sub idcheck {
   my %missing = ();
   foreach my $node ($self->findnodes("//*[\@xml:id]")) {
     my $id = $node->getAttribute('xml:id');
-    $dups{$id} = 1 if $idcache{$id};
+    $dups{$id}    = 1 if $idcache{$id};
     $idcache{$id} = 1; }
   foreach my $id (keys %{ $$self{idcache} }) {
     $missing{$id} = 1 unless $idcache{$id}; }
-  LaTeXML::Post::Warn("unexpected", 'ids', undef,
+  Warn("unexpected", 'ids', undef,
     "IDs were duplicated in cache for " . $self->siteRelativeDestination,
     join(',', keys %dups))
     if keys %dups;
-  LaTeXML::Post::Warn("expected", 'ids', undef, "IDs were cached for " . $self->siteRelativeDestination
+  Warn("expected", 'ids', undef, "IDs were cached for " . $self->siteRelativeDestination
       . " but not in document",
     join(',', keys %missing))
     if keys %missing;
@@ -958,6 +1083,7 @@ sub getQName {
 # we have to orient everything towards appending.
 # In particular, see the perversity in the following few methods.
 sub addNodes {
+  no warnings 'recursion';
   my ($self, $node, @data) = @_;
   foreach my $child (@data) {
     if (ref $child eq 'ARRAY') {
@@ -974,7 +1100,7 @@ sub addNodes {
       else {
         my ($prefix, $localname) = $tag =~ /^(.*):(.*)$/;
         my $nsuri = $prefix && $$self{namespaces}{$prefix};
-        LaTeXML::Post::Warn('expected', 'namespace', undef, "No namespace on '$tag'") unless $nsuri;
+        Warn('expected', 'namespace', undef, "No namespace on '$tag'") unless $nsuri;
         my $new;
         if (ref $node eq 'LibXML::XML::Document') {
           $new = $node->createElementNS($nsuri, $localname);
@@ -1017,7 +1143,7 @@ sub addNodes {
           my $atype = $attr->nodeType;
           if ($atype == XML_ATTRIBUTE_NODE) {
             my $key = $attr->nodeName;
-            if ($key =~ /^_/) { }    # don't copy internal attributes
+            if    ($key =~ /^_/) { }     # don't copy internal attributes
             elsif ($key eq 'xml:id') {
               my $value = $attr->getValue;
               my $old;
@@ -1041,7 +1167,7 @@ sub addNodes {
         $node->appendTextNode($child->textContent); }
     }
     elsif (ref $child) {
-      LaTeXML::Post::Warn('misdefined', $child, undef, "Dont know how to add $child to $node; ignoring"); }
+      Warn('misdefined', $child, undef, "Dont know how to add $child to $node; ignoring"); }
     elsif (defined $child) {
       $node->appendTextNode($child); } }
   return; }
@@ -1052,7 +1178,7 @@ sub removeNodes {
   my ($self, @nodes) = @_;
   foreach my $node (@nodes) {
     my $ref = ref $node;
-    if (!$ref) { }
+    if    (!$ref) { }
     elsif ($ref eq 'ARRAY') {
       my ($t, $a, @n) = @$node;
       if (my $id = $$a{'xml:id'}) {
@@ -1074,7 +1200,7 @@ sub preremoveNodes {
   my ($self, @nodes) = @_;
   foreach my $node (@nodes) {
     my $ref = ref $node;
-    if (!$ref) { }
+    if    (!$ref) { }
     elsif ($ref eq 'ARRAY') {
       my ($t, $a, @n) = @$node;
       if (my $id = $$a{'xml:id'}) {
@@ -1130,13 +1256,14 @@ sub prependNodes {
 sub cloneNode {
   my ($self, $node, $idsuffix, %options) = @_;
   return $node unless ref $node;
+  return $node if ref $node eq 'ARRAY'; # Should we deep clone if we get an array? Just return for now
   my $copy    = $node->cloneNode(1);
   my $nocache = $options{nocache};
 ####  $idsuffix = '' unless defined $idsuffix;
   # Find all id's defined in the copy and change the id.
   my %idmap = ();
   foreach my $n ($self->findnodes('descendant-or-self::*[@xml:id]', $copy)) {
-    my $id = $n->getAttribute('xml:id');
+    my $id    = $n->getAttribute('xml:id');
     my $newid = $self->uniquifyID($id, $idsuffix);
     $idmap{$id} = $newid;
     $self->recordID($newid => $n) unless $nocache;
@@ -1193,6 +1320,7 @@ sub markXMNodeVisibility {
   return; }
 
 sub markXMNodeVisibility_aux {
+  no warnings 'recursion';
   my ($self, $node, $cvis, $pvis) = @_;
   return unless $node;
   my $qname = $self->getQName($node);
@@ -1250,14 +1378,14 @@ sub trimChildNodes {
   elsif (!ref $node) {
     return ($node); }
   elsif (my @children = $node->childNodes) {
-    if ($children[0]->nodeType == XML_TEXT_NODE) {
+    if ($children[0] && $children[0]->nodeType == XML_TEXT_NODE) {
       my $s = $children[0]->data;
       $s =~ s/^\s+//;
       if ($s) {
         $children[0]->setData($s); }
       else {
         shift(@children); } }
-    if ($children[-1]->nodeType == XML_TEXT_NODE) {
+    if ($children[-1] && $children[-1]->nodeType == XML_TEXT_NODE) {
       my $s = $children[-1]->data;
       $s =~ s/\s+$//;
       if ($s) {
@@ -1302,18 +1430,32 @@ sub findNodeByID {
   my $node = $$self{idcache}{$id};
   return $$self{idcache}{$id}; }
 
+# If $branch given, should be 'content' or 'presentation'
 sub realizeXMNode {
-  my ($self, $node) = @_;
-  if ($self->getQName($node) eq 'ltx:XMRef') {
+  my ($self, $node, $branch) = @_;
+  if ($branch) {
+    while ($node) {
+      my $qname = $self->getQName($node);
+      if ($qname eq 'ltx:XMRef') {
+        my $id = $node->getAttribute('idref');
+        if (my $realnode = $self->findNodeByID($id)) {
+          $node = $realnode; }
+        else {
+          Error('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
+          return; } }
+      elsif ($qname eq 'ltx:XMDual') {
+        my ($content, $presentation) = element_nodes($node);
+        $node = ($branch eq 'content' ? $content : $presentation); }
+      else {
+        return $node; } } }
+  elsif ($self->getQName($node) eq 'ltx:XMRef') {
     my $id = $node->getAttribute('idref');
     if (my $realnode = $self->findNodeByID($id)) {
-      #print STDERR "REALIZE $id => $realnode\n";
       return $realnode; }
     else {
-      Fatal('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
+      Error('expected', 'id', undef, "Cannot find a node with xml:id='$id'");
       return; } }
-  else {
-    return $node; } }
+  return $node; }
 
 sub uniquifyID {
   my ($self, $baseid, $suffix) = @_;
@@ -1380,11 +1522,13 @@ sub adjust_latexml_doctype {
 sub cacheLookup {
   my ($self, $key) = @_;
   $self->openCache;
+  $key = Encode::encode_utf8($key) if $key;
   return $$self{cache}{$key}; }
 
 sub cacheStore {
   my ($self, $key, $value) = @_;
   $self->openCache;
+  $key = Encode::encode_utf8($key) if $key;
   if (defined $value) {
     $$self{cache}{$key} = $value; }
   else {
