@@ -39,7 +39,7 @@ our $LATEXCMD = 'latex';    #(or elatex) [ CONFIGURABLE? Encoded in PI?]
 #   source         : (dir)
 #   magnification  : typically something like 1.33333, but you may want bigger
 #   maxwidth       : maximum page width, in pixels (whenever line breaking is possible)
-#   dpi            : assumed DPI for the target medium (default 96)
+#   DPI            : assumed DPI for the target medium (default 100)
 #   background     : color of background (for anti-aliasing, since it is made transparent)
 #   imagetype      : typically 'png' or 'gif'.
 sub new {
@@ -47,14 +47,14 @@ sub new {
   my $self = $class->SUPER::new(%options);
   $$self{magnification} = $options{magnification} || 1.33333;
   $$self{maxwidth}      = $options{maxwidth}      || 800;
-  $$self{dpi}           = $options{dpi}           || 96;
+  $$self{DPI}           = $options{DPI}           || 100;
   $$self{background}    = $options{background}    || "#FFFFFF";
   $$self{imagetype}     = $options{imagetype}     || 'png';
 
   # Parameters for separating the clipping box from the
   # desired padding between image edge and "ink"
   $$self{padding} = $options{padding} || 2;    # pixels
-        # amount of extra space (+padding) to put between object & rules for clipping
+      # amount of extra space (+padding) to put between object & rules for clipping
   $$self{clippingfudge} = 3;       # px
   $$self{clippingrule}  = 0.90;    # pixels (< 1 to avoid antialiasing..?)
 
@@ -73,7 +73,7 @@ sub new {
 
   # Parameterize according to the selected dvi-to-whatever processor.
   my $mag = int($$self{magnification} * 1000);
-  my $dpi = int($$self{dpi} * $$self{magnification});
+  my $dpi = int($$self{DPI} * $$self{magnification});
   # Unfortunately, each command has incompatible -o option to name the output file.
   # Note that the formatting char used, '%', has to be doubled on Windows!!
   # Maybe that's only true when it looks like an environment variable?
@@ -81,19 +81,14 @@ sub new {
   #  my $fmt = ($^O eq 'MSWin32' ? '%%' : '%');
   my $fmt = '%';
   if ($$self{use_dvisvgm}) {
-    # img name uses 2 digits for page; what happens at 100?
     ##    $$self{dvicmd}             = "dvisvgm --page=1- --bbox=min --mag=$mag -o imgx-${fmt}p";
     # dvisvgm currently creates glyph descriptions w/ unicode attribute having the wrong codepoint
     # firefox, chromium use this codepoint instead of the glyph "drawing"
     # a later version of dvisvgm should do better at synthesizing the unicode?
     # but for now, we'll use --no-fonts, which creates glyph drawings rather than "glyphs"
     # Also, increase the bounding box from min by 1pt
-    # Also, note incompatible change in the -o option after version 1.6;
-    # it now can take a number of digits (%3p) for pages, BUT old version produces imgx-%3p.svg!!
-    # So, simply use %p, let new version do 1,..,9,10,...
-    # Recovery code below!
-    $$self{dvicmd} = "dvisvgm --page=1- --bbox=1pt --scale=$$self{magnification} --no-fonts -o imgx-${fmt}p";
-    $$self{dvicmd_output_name} = 'imgx-%02d.svg';
+    $$self{dvicmd} = "dvisvgm --page=1- --bbox=1pt --scale=$$self{magnification} --no-fonts -o imgx-${fmt}3p";
+    $$self{dvicmd_output_name} = 'imgx-%03d.svg';
     $$self{dvicmd_output_type} = 'svg';
     $$self{frame_output}       = 0; }
   elsif ($$self{use_dvipng}) {
@@ -163,7 +158,7 @@ sub format_tex {
 # This is the default
 sub setTeXImage {
   my ($self, $doc, $node, $path, $width, $height, $depth) = @_;
-  $node->setAttribute('imagesrc',    $path);
+  $node->setAttribute('imagesrc',    pathname_to_url($path));
   $node->setAttribute('imagewidth',  $width);
   $node->setAttribute('imageheight', $height);
   $node->setAttribute('imagedepth',  $depth) if defined $depth;
@@ -181,8 +176,8 @@ sub cleanTeX {
   if ($tex =~ s/^\s*(\\displaystyle|\\textstyle|\\scriptstyle|\\scriptscriptstyle)\s*//) {
     $style = $1; }
   $tex =~ s/^(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*//; # Trim leading spacing (especially negative!)
-  $tex =~ s/(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*$//; # and trailing spacing
-       # Strip comments, but watchout for \% (or more exactly, an ODD number of \)
+  $tex =~ s/(?:\\\s*,|\\!\s*|\\>\s*|\\;\s*|\\:\s*|\\ \s*|\\\/\s*)*$//;    # and trailing spacing
+      # Strip comments, but watchout for \% (or more exactly, an ODD number of \)
   $tex =~ s/(?<!\\)((?:\\\\)*)\%[^\n]*\n/$1/gs;
   $tex = $style . ' ' . $tex if $style;    # Put back the style, if any
   return $tex; }
@@ -209,11 +204,7 @@ sub generateImages {
   my $jobname  = "ltxmlimg";
   my $orig_cwd = pathname_cwd();
   my $sep      = $Config::Config{path_sep};
-  local $ENV{TEXINPUTS} = join($sep, '.', $doc->getSearchPaths,
-    pathname_concat(pathname_installation(), 'texmf'),
-    ($ENV{TEXINPUTS} || $sep));
-
-  my %table = ();
+  my %table    = ();
 
   # === Get the desired nodes, extract the set of unique tex strings,
   #     noting which need processing.
@@ -253,23 +244,29 @@ sub generateImages {
       next if -f pathname_absolute($1, $destdir); }
     push(@pending, $table{$key}); }
 
-  NoteProgress(" [$nuniq unique; " . scalar(@pending) . " new]");
+  Debug("LaTeXImages: $nuniq unique; " . scalar(@pending) . " new") if $LaTeXML::DEBUG{images};
   if (@pending) {    # if any images need processing
         # Create working directory; note TMPDIR attempts to put it in standard place (like /tmp/)
-    File::Temp->safe_level(File::Temp::HIGH);
+    File::Temp->safe_level(File::Temp::MEDIUM);
     my $workdir = File::Temp->newdir("LaTeXMLXXXXXX", TMPDIR => 1);
+    # Obtain the search paths
+    my @searchpaths       = $doc->getSearchPaths;
+    my $installation_path = pathname_installation();
     # === Generate the LaTeX file.
     my $texfile = pathname_make(dir => $workdir, name => $jobname, type => 'tex');
     my $TEX;
     if (!open($TEX, '>', $texfile)) {
       Error('I/O', $texfile, undef, "Cant write to '$texfile'", "Response was: $!");
       return $doc; }
+    my ($pre_preamble, $add_to_body) = $self->pre_preamble($doc);
     binmode $TEX, ':encoding(UTF-8)';
-    print $TEX $self->pre_preamble($doc);
+    print $TEX $pre_preamble if $pre_preamble;
     print $TEX "\\makeatletter\n";
     print $TEX $self->preamble($doc) . "\n";
     print $TEX "\\makeatother\n";
     print $TEX "\\begin{document}\n";
+    print $TEX $add_to_body if $add_to_body;
+
     foreach my $entry (@pending) {
 ##      print TEX "\\fbox{$$entry{tex}}\\clearpage\n"; }
       print $TEX "$$entry{tex}\\clearpage\n"; }
@@ -280,20 +277,27 @@ sub generateImages {
     # (keep the command simple so it works in Windows)
     pathname_chdir($workdir);
     my $ltxcommand = "$LATEXCMD $jobname > $jobname.ltxoutput";
-    my $ltxerr     = system($ltxcommand);
+    my $ltxerr;
+    {
+      local $ENV{TEXINPUTS} = join($sep, '.', @searchpaths,
+        pathname_concat($installation_path, 'texmf'),
+        ($ENV{TEXINPUTS} || $sep));
+      $ltxerr = system($ltxcommand);
+    }
     pathname_chdir($orig_cwd);
 
     # Sometimes latex returns non-zero code, even though it apparently succeeded.
     # And sometimes it doesn't produce a dvi, even with 0 return code?
     if (($ltxerr != 0) || (!-f "$workdir/$jobname.dvi")) {
-      my $preserve = $$LaTeXML::POST{verbosity} > 0;    # preserve junk when verbosity high.
-      $workdir->unlink_on_destroy(0) if $preserve;
+      $workdir->unlink_on_destroy(0) if $LaTeXML::DEBUG{images};    # Preserve junk
       Error('shell', $LATEXCMD, undef,
         "LaTeX command '$ltxcommand' failed",
-        ($ltxerr == 0 ? "No dvi file generated"     : "returned code $ltxerr (!= 0): $@"),
-        ($preserve    ? "See $workdir/$jobname.log" : "Re-run with --verbose to see TeX log"));
+        ($ltxerr == 0 ? "No dvi file generated" : "returned code $ltxerr (!= 0): $@"),
+        ($LaTeXML::DEBUG{images}
+          ? "See $workdir/$jobname.log"
+          : "Re-run with --debug=images to see TeX log"));
       return $doc; }
-    $workdir->unlink_on_destroy(0) if $$LaTeXML::POST{verbosity} > 2;
+    ### $workdir->unlink_on_destroy(0) if $LaTeXML::DEBUG{images}; # preserve ALL junk!?!?!
 
     # Extract dimensions (width x height+depth) from each image from log file.
     my @dimensions = ();
@@ -311,7 +315,13 @@ sub generateImages {
     # === Run dvicmd to extract individual png|postscript files.
     pathname_chdir($workdir);
     my $dvicommand = "$$self{dvicmd} $jobname.dvi > $jobname.dvioutput";
-    my $dvierr     = system($dvicommand);
+    my $dvierr;
+    {
+      local $ENV{TEXINPUTS} = join($sep, '.', @searchpaths,
+        pathname_concat($installation_path, 'texmf'),
+        ($ENV{TEXINPUTS} || $sep));
+      $dvierr = system($dvicommand);
+    }
     pathname_chdir($orig_cwd);
 
     if ($dvierr != 0) {
@@ -320,14 +330,10 @@ sub generateImages {
         "Response was: $!");
       return $doc; }
     # === Convert each image to appropriate type and put in place.
-    my $pixels_per_pt = $$self{magnification} * $$self{dpi} / 72.27;
+    my $pixels_per_pt = $$self{magnification} * $$self{DPI} / 72.27;
     my ($index, $ndigits) = (0, 1 + int(log($doc->cacheLookup((ref $self) . ':_max_image_') || 1) / log(10)));
     foreach my $entry (@pending) {
       my $src = "$workdir/" . sprintf($$self{dvicmd_output_name}, ++$index);
-      # Recovery patchup for incompatible change to dvisvgm!
-      if (($index == 1) && (!-f $src) && (-f "$workdir/imgx-1.svg")) {
-        $$self{dvicmd_output_name} = 'imgx-%d.svg';
-        $src = "$workdir/imgx-1.svg"; }
       if (-f $src) {
         my @dests = @{ $$entry{dest} };
         push(@dests, $self->generateResourcePathname($doc, $$entry{nodes}[0], undef, $$self{imagetype}))
@@ -345,7 +351,6 @@ sub generateImages {
           my $d = int(0.5 + ($dd || 0) + $$self{padding});
           if ((($w == 1) && ($ww > 1)) || (($h == 1) && ($hh > 1))) {
             Warn('expected', 'image', undef, "Image for '$$entry{tex}' was cropped to nothing!"); }
-#          print STDERR "\nFor key='$$entry{key}': Image[$index] '$dest' $$entry{tex} $ww x $hh + $dd ==> $w x $h \\ $d\n";
           $doc->cacheStore($$entry{key}, "$dest;$w;$h;$d"); }
       }
       else {
@@ -377,11 +382,22 @@ sub pre_preamble {
   my $packages        = '';
   my $dest            = $doc->getDestination;
   my $description     = ($dest ? "% Destination $dest" : "");
-  my $pts_per_pixel   = 72.27 / $$self{dpi} / $$self{magnification};
+  my $pts_per_pixel   = 72.27 / $$self{DPI} / $$self{magnification};
+  my %loaded_check    = ();
+
   foreach my $pkgdata (@classdata) {
     my ($package, $package_options) = @$pkgdata;
-    $package_options = "[$package_options]" if $package_options && ($package_options !~ /^\[.*\]$/);
-    $packages .= "\\usepackage$package_options\{$package\}\n"; }
+    next if $loaded_check{$package};
+    $loaded_check{$package} = 1;
+    if ($oldstyle) {
+      next if $package =~ /latexml/;    # some packages are incompatible.
+      $packages .= "\\RequirePackage{$package}\n"; }
+    else {
+      if ($package eq 'english') {
+        $packages .= "\\usepackage[english]{babel}\n"; }
+      else {
+        $package_options = "[$package_options]" if $package_options && ($package_options !~ /^\[.*\]$/);
+        $packages .= "\\usepackage$package_options\{$package}\n"; } } }
 
   my $w   = ceil($$self{maxwidth} * $pts_per_pixel);                      # Page Width in points.
   my $gap = ($$self{padding} + $$self{clippingfudge}) * $pts_per_pixel;
@@ -389,17 +405,32 @@ sub pre_preamble {
     ? $$self{clippingrule} * $pts_per_pixel    # clipping box thickness in points.
     : 0);                                      # NO clipping box!
   my $preambles = $self->find_preambles($doc);
+  # Some classes are too picky: thanks but no thanks
+  my $result_add_to_body = "\\makeatletter\\thispagestyle{empty}\\pagestyle{empty}\n";
+  # neutralize caption macros
+  $result_add_to_body .= "\\let\\\@\@toccaption\\\@gobble\n\\let\\\@\@caption\\\@gobble\n"
+    . "\\let\\cite\\\@gobble\n\\def\\\@\@bibref#1#2#3#4{}\n";
+  $result_add_to_body .= "\\renewcommand{\\cite}[2][]{}\n" unless $oldstyle;
+  # class-specific conditions:
+  if ($class =~ /^JHEP$/i) {
+    $class = 'article'; }
+  elsif ($class =~ /revtex/) {
+    # Careful with revtex4
+    $result_add_to_body .= "\\\@ifundefined{\@author\@def}{\\author{}}{}\n"; }
+  # when are the empty defaults needed?
+  if ($class ne 'article') {
+    $result_add_to_body .= "\\title{}\\date{}\n"; }
+  $result_add_to_body .= "\\makeatother\n";
 
-  return <<"EOPreamble";
+  my $result_preamble = <<"EOPreamble";
 \\batchmode
 \\def\\inlatexml{true}
-$documentcommand$class_options\{$class\}
+$documentcommand$class_options\{$class}
 $description
 $packages
 \\makeatletter
 \\setlength{\\hoffset}{0pt}\\setlength{\\voffset}{0pt}
 \\setlength{\\textwidth}{${w}pt}
-\\thispagestyle{empty}\\pagestyle{empty}\\title{}\\author{}\\date{}
 \\newcount\\lxImageNumber\\lxImageNumber=0\\relax
 \\newbox\\lxImageBox
 \\newdimen\\lxImageBoxSep
@@ -438,7 +469,8 @@ $packages
 $preambles
 \\makeatother
 EOPreamble
-}
+
+  return ($result_preamble, $result_add_to_body); }
 
 #======================================================================
 # Converting the postscript images to gif/png/whatever
@@ -454,7 +486,7 @@ sub convert_image {
   my ($self, $doc, $src, $dest) = @_;
   my ($bg, $fg) = ($$self{background}, 'black');
 
-  my $image = image_object(antialias => 'True', background => $bg, density => $$self{dpi});
+  my $image = image_object(antialias => 'True', background => $bg, density => $$self{DPI});
   my $err   = $image->Read($$self{dvicmd_output_type} . ':' . $src);
   if ($err) {
     Warn('imageprocessing', 'read', undef,
@@ -469,16 +501,16 @@ sub convert_image {
   $image->Border(width => 1, height => 1, fill => $bg);
   $image->Trim(fuzz => '75%');    # Fuzzy, to trim the gray tabs, as well!!
   $image->Set(fuzz => '0%');      # But, be SURE to RESET fuzz!! (It is NOT an "argument"!!!)
-       # [Also, be CAREFULL of ImageMagick's Floodfill variants, they sometimes go wild]
+      # [Also, be CAREFULL of ImageMagick's Floodfill variants, they sometimes go wild]
 
   # Finally, shave off the rule & fudge padding.
   my $fudge = int(0.5 + $$self{clippingfudge} + $$self{clippingrule});
   $image->Shave(width => $fudge, height => $fudge);
 
   my ($w, $h) = $image->Get('width', 'height');    # Get final image size
-         # ImageMagick tries to manage a "virtual" image within the image data,
-         # (whatever that means)
-         # This resets it back to the origin which avoids confusion, all 'round!
+      # ImageMagick tries to manage a "virtual" image within the image data,
+      # (whatever that means)
+      # This resets it back to the origin which avoids confusion, all 'round!
   $image->Set(page => "${w}x${h}+0+0");
 
   # Ideally, we'd make the alpha exactly opposite the ink, rather than merely 1 bit
@@ -486,8 +518,7 @@ sub convert_image {
   #  $image = $image->Fx(expression=>'(3.0-r+g+b)/3.0', channel=>'alpha');
   $image->Transparent(color => $bg);
 
-  NoteProgressDetailed(" [Converting $src => $dest ($w x $h)]");
-
+  Debug("LaTeXImages: Converting $src => $dest ($w x $h)") if $LaTeXML::DEBUG{images};
   # Workaround bad png detection(?) in ImageMagick 6.6.5 (???)
   if ($$self{imagetype} eq 'png') {
     $dest = "png32:$dest"; }
