@@ -34,25 +34,39 @@ sub new {
 
 sub toProcess {
   my ($self, $doc) = @_;
-  return $doc->findnode('//ltx:index'); }
+  return $doc->findnodes('//ltx:index[not(ltx:indexlist)] | //ltx:glossary[not(ltx:glossarylist)]'); }
 
 sub process {
-  my ($self, $doc, $index) = @_;
-  my @indices = ($doc);
+  my ($self, $doc, @lists) = @_;
+  my @newdocs = ();
   $doc->addDate();
-  my ($tree, $allphrases) = $self->build_tree($doc, $index);
-  if ($tree) {
-    if ($$self{split}) {
-      @indices = map { $self->rescan($_) }
-        $self->makeSubCollectionDocuments($doc, $index,
-        map { ($_ => $self->makeIndexList($doc, $allphrases, $$tree{subtrees}{$_})) }
-          keys %{ $$tree{subtrees} }); }
-    else {
-      $doc->addNodes($index, $self->makeIndexList($doc, $allphrases, $tree));
-      @indices = ($self->rescan($doc)); } }
-  return @indices; }
+  foreach my $list (@lists) {
+    my $tag = $doc->getQName($list);
+    if ($tag eq 'ltx:index') {
+      my ($tree, $allphrases) = $self->build_tree($doc, $list);
+      if ($tree) {
+        if ($$self{split}) {
+          push(@newdocs, $self->makeSubCollectionDocuments($doc, $list,
+              map { ($_ => $self->makeIndexList($doc, $allphrases, $$tree{subtrees}{$_})) }
+                keys %{ $$tree{subtrees} })); }
+        else {
+          $doc->addNodes($list, $self->makeIndexList($doc, $allphrases, $tree)); } } }
+    elsif ($tag eq 'ltx:glossary') {
+      if (my @entries = $self->getGlossaryEntries($doc, $list)) {
+        if ($$self{split}) {    # Separate by initial.
+          my $split = {};
+          foreach my $entry (@entries) {
+            push(@{ $$split{ $$entry{initial} } }, $entry); }
+          push(@newdocs, $self->makeSubCollectionDocuments($doc, $list,
+              map { ($_ => $self->makeGlossaryList($doc, @{ $$split{$_} })) }
+                sort keys %$split)); }
+        else {
+          $doc->addNodes($list, $self->makeGlossaryList($doc, @entries)); } } } }
+  my @docs = map { $self->rescan($_); } (@newdocs ? @newdocs : ($doc));
+  return @docs; }
 
 # ================================================================================
+# Index
 # Data generated:
 #  $tree : tree representation of the index.
 #  $allphrases : used for inferring the connection from see-also phrases to normal index entries.
@@ -63,17 +77,17 @@ sub build_tree {
   my $defaultlistname = 'idx';
   my $listname        = 'idx';    # Eventually customizable for different indices?
   if (my @keys = grep { /^INDEX:/ } $$self{db}->getKeys) {
-    NoteProgress(" [" . scalar(@keys) . " entries]");
+    NoteLog("MakeIndex: " . scalar(@keys) . " entries");
 
-    my $id = $index->getAttribute('xml:id');
+    my $id         = $index->getAttribute('xml:id');
     my $allphrases = {};          # Keep a hash of all phrase textContent=>id encountered (for seealso)
-    my $tree = { subtrees => {}, referrers => {}, id => $id, parent => undef };
+    my $tree       = { subtrees => {}, referrers => {}, id => $id, parent => undef };
     foreach my $key (@keys) {
       my $entry   = $$self{db}->lookup($key);
       my $phrases = $entry->getValue('phrases');
       my @phrases = @$phrases;
       if (($entry->getValue('inlist') || $defaultlistname) ne $listname) {
-        next; }                   # Skip any not in this list
+        next; }    # Skip any not in this list
       if (!scalar(@phrases)) {
         Warn('expected', $key, undef, "Missing phrases in indexmark: '$key'");
         next; }
@@ -174,7 +188,7 @@ my $GREEK_RE = '(' . join('|', sort keys %GREEK_ASCII_MAP) . ')';
 sub getIndexKeyID {
   my ($key) = @_;
   $key =~ s/^\s+//s; $key =~ s/\s+$//s;    # Trim leading/trailing, in any case
-        # We don't want accented chars (do we?) but we need to decompose the accents!
+      # We don't want accented chars (do we?) but we need to decompose the accents!
   $key = NFD($key);
   $key =~ s/$GREEK_RE/$GREEK_ASCII_MAP{$1}/eg;
   $key = unidecode($key);
@@ -317,7 +331,7 @@ sub seealsoSearch {
 sub seealsoSearch_rec {
   my ($doc, $allphrases, $contexttree, @parts) = @_;
   my ($link, @links);
-  if (scalar(@parts) < 1) { return (); }
+  if    (scalar(@parts) < 1) { return (); }
   elsif (scalar(@parts) < 3) {
     # Single term? (w/ possible trailing punct) just look it up
     if ($link = lookupSeealsoPhrase($doc, $allphrases, $contexttree, $parts[0])) {
@@ -352,10 +366,10 @@ sub lookupSeealsoPhrase {
   my ($doc, $allphrases, $contexttree, $pair) = @_;
   my ($phrase, @xml) = @$pair;
   # concoct various phrases to search for
-  my $pnc   = $phrase; $pnc =~ s/,\s*/ /sg;        # Ignore punct?
-  my $ps    = $phrase; $ps =~ s/(\w+)s\b/$1/sg;    # Ignore plurals?
-  my $psnc  = $ps;     $psnc =~ s/,\s*/ /sg;       # Ignore punct AND plurals?
-  my $pnlvl = $phrase; $pnlvl =~ s/,\s*/./sg;      # Convert punct to levels?
+  my $pnc   = $phrase; $pnc   =~ s/,\s*/ /sg;         # Ignore punct?
+  my $ps    = $phrase; $ps    =~ s/(\w+)s\b/$1/sg;    # Ignore plurals?
+  my $psnc  = $ps;     $psnc  =~ s/,\s*/ /sg;         # Ignore punct AND plurals?
+  my $pnlvl = $phrase; $pnlvl =~ s/,\s*/./sg;         # Convert punct to levels?
   foreach my $trial ($phrase, lc($phrase),
     $pnc,   lc($pnc),
     $ps,    lc($ps),
@@ -387,7 +401,7 @@ sub seealsoPartition {
   while (@parts) {
     my $next    = shift(@parts);
     my $prev_is = ($result[-1][0] =~ /^,?\s*(?:,|\.|\s+|\band\s+also|\band|\bor)\s*$/);
-    my $next_is = ($$next[0] =~ /^(?:,|\.|\s+|and\b|or\b)/);
+    my $next_is = ($$next[0]      =~ /^(?:,|\.|\s+|and\b|or\b)/);
     # If either BOTH or NEITHER prev & next are delimiters, combine them.
     if (!($prev_is xor $next_is)) {
       my ($k, @x) = @$next;
@@ -435,6 +449,46 @@ sub seealsoPartition_aux {
   } }
   return @result; }
 
+# ================================================================================
+# Glosssary
+# ================================================================================
+
+sub getGlossaryEntries {
+  my ($self, $doc, $glossary) = @_;
+  my $lists = $glossary->getAttribute('lists') || '';
+  return () unless $lists;
+  my $glossary_id = $glossary->getAttribute('xml:id') || '';
+  my %lists       = map { $_ => 1; } split(',', $lists);
+  my @entries     = ();
+  foreach my $gkey (grep { /^GLOSSARY:/ } $$self{db}->getKeys) {
+    my ($ignore, $list, $key) = split(':', $gkey);
+    next unless $lists{$list};
+    my $gitem = $$self{db}->lookup($gkey);
+    my $refs  = $gitem->getValue('referrers');
+    next unless $refs && %{$refs};
+    my $term    = $gitem->getValue('phrase:name');
+    my $desc    = $gitem->getValue('phrase:description');
+    my $sortkey = $gitem->getValue('phrase:sort') || $key;
+    my $initial = ($sortkey =~ /^([a-zA-Z])/ ? uc($1) : '*');
+    my $id      = $glossary_id . '.' . $key;
+    push(@entries,
+      { initial => $initial, sortkey => $sortkey,
+        formatted => ['ltx:glossaryentry', { lists => $lists, 'xml:id' => $id, key => $key },
+          ['ltx:glossaryphrase', { role => 'label', key => $key },    # what key??
+            (map { $doc->cloneNode($_, 'glo') } $doc->trimChildNodes($term))],
+          ['ltx:glossaryphrase', { role => 'definition' },
+            (map { $doc->cloneNode($_, 'glo') } $doc->trimChildNodes($desc))
+          ]] });
+  }
+  return @entries; }
+
+sub makeGlossaryList {
+  my ($self, $doc, @entries) = @_;
+  my %hash = map { ($$_{sortkey} => $_) } @entries;
+  my @keys = $doc->unisort(keys %hash);
+  return ['ltx:glossarylist', {},
+    map { $hash{$_}{formatted} } @keys]; }
+
 # ================================================================================
 sub conjoin {
   my (@items) = @_;
