@@ -46,6 +46,7 @@ sub new {
   $self->registerHandler('ltx:equation'      => \&labelled_handler);
   $self->registerHandler('ltx:equationgroup' => \&labelled_handler);
   $self->registerHandler('ltx:item'          => \&labelled_handler);
+  $self->registerHandler('ltx:listingline'   => \&labelled_handler);
   $self->registerHandler('ltx:anchor'        => \&anchor_handler);
   $self->registerHandler('ltx:note'          => \&note_handler);
 
@@ -56,6 +57,7 @@ sub new {
   $self->registerHandler('ltx:glossarydefinition' => \&glossaryentry_handler);
   $self->registerHandler('ltx:ref'                => \&ref_handler);
   $self->registerHandler('ltx:bibref'             => \&bibref_handler);
+  $self->registerHandler('ltx:glossaryref'        => \&glossaryref_handler);
 
   $self->registerHandler('ltx:navigation' => \&navigation_handler);
   $self->registerHandler('ltx:rdf'        => \&rdf_handler);
@@ -263,8 +265,7 @@ sub addCommon {
       else {
         $key = 'tag:' . $role; } }
     else {
-      $key = 'frefnum'; }
-    ###      $key = 'refnum'; }        # ???
+      $key = 'refnum'; }
     $props{$key} = $self->cleanNode($doc, $tagnode); }
   return %props; }
 
@@ -379,7 +380,6 @@ sub bibref_handler {
       my @lists = (($l ? split(/\s+/, $l) : ()), 'bibliography');
       foreach my $bibkey (split(',', $keys)) {
         if ($bibkey) {
-          $bibkey = lc($bibkey);         # NOW we downcase!
           foreach my $list (@lists) {    # Records a *reference* to a bibkey! (for each list)
             my $entry = $$self{db}->register("BIBLABEL:$list:$bibkey");
             $entry->noteAssociation(referrers => $parent_id); } } } } }
@@ -387,6 +387,16 @@ sub bibref_handler {
   $self->default_handler($doc, $node, $tag, $parent_id);
   return; }
 
+sub glossaryref_handler {
+  my ($self, $doc, $node, $tag, $parent_id) = @_;
+  my $list = $node->getAttribute('inlist');
+  my $key  = $node->getAttribute('key');
+  if ($key && $list) {
+    my $entry = $$self{db}->register(join(':', 'GLOSSARY', $list, $key));
+    $entry->noteAssociation(referrers => $parent_id); }
+  $self->default_handler($doc, $node, $tag, $parent_id);
+  return; }
+
 # Note that index entries get stored in simple form; just the terms & location.
 # They will be turned into a tree, sorted, possibly permuted, whatever, by MakeIndex.
 # [the only content of indexmark should be un-marked up(?) don't recurse]
@@ -409,33 +419,32 @@ sub indexmark_handler {
   return; }
 
 # This handles glossaryentry or glossarydefinition
+#  * glossarydefinition is the original description of the item,
+#     randomly (perhaps) located in the document
+#  * glossaryentry is the duplicate(?) of the above
+#     located within a glossarylist.
+# Probably, the latter is what should be referenced
 sub glossaryentry_handler {
   my ($self, $doc, $node, $tag, $parent_id) = @_;
-  my $id = $node->getAttribute('xml:id');
+  # Only register the id for the glossaryentry (in the glossarylist) as desired target
+  my $id = ($tag eq 'ltx:glossaryentry' ? $node->getAttribute('xml:id') : undef);
   my $p;
   my $lists = $node->getAttribute('inlist') ||
     (($p = $doc->findnode('ancestor::ltx:glossarylist[@lists] | ancestor::ltx:glossary[@lists]', $node))
     && $p->getAttribute('lists'))
     || 'glossary';
-  my $key = $node->getAttribute('key');
-  # Get the actual phrases, and any see_also phrases (if any)
-  # Do these need ->cleanNode ???
+  my $key     = $node->getAttribute('key');
   my @phrases = $doc->findnodes('ltx:glossaryphrase', $node);
   # Create an entry for EACH list (they could be distinct definitions)
   foreach my $list (split(/\s+/, $lists)) {
     my $gkey  = join(':', 'GLOSSARY', $list, $key);
     my $entry = $$self{db}->lookup($gkey) || $$self{db}->register($gkey);
     $entry->setValues(map { ('phrase:' . ($_->getAttribute('role') || 'label') => $_) } @phrases);
-    $entry->noteAssociation(referrers => $parent_id => ($node->getAttribute('style') || 'normal'));
     $entry->setValues(id => $id) if $id; }
 
   if ($id) {
-    $$self{db}->register("ID:$id", id => orNull($id), type => orNull($tag), parent => orNull($parent_id),
-      labels   => orNull($self->noteLabels($node)),
-      location => orNull($doc->siteRelativeDestination),
-      pageid   => orNull($self->pageID($doc)),
-      fragid   => orNull($self->inPageID($doc, $node))); }
-  # Scan content, since could contain other interesting stuff...
+    $$self{db}->register("ID:$id",
+      $self->addCommon($doc, $node, $tag, $parent_id)); }
   $self->scanChildren($doc, $node, $id || $parent_id);
   return; }
 
@@ -450,10 +459,7 @@ sub bibitem_handler {
   my ($self, $doc, $node, $tag, $parent_id) = @_;
   my $id = $node->getAttribute('xml:id');
   if ($id) {
-    # NOTE: We didn't downcase the key when we created the bib file
-    # BUT, we're going to index it in the ObjectDB by the downcased name!!!
     my $key = $node->getAttribute('key');
-    $key = lc($key) if $key;
     my $bib = $doc->findnode('ancestor-or-self::ltx:bibliography', $node);
     # Probably should only be one list, but just in case?
     my @lists = split(/\s+/, ($bib && $bib->getAttribute('lists')) || 'bibliography');
